@@ -64,12 +64,12 @@ class KeepalivedLvs(udp_listener_base.UdpListenerApiServerBase):
                 assert True is os.path.exists(script_dir)
                 assert True is os.path.exists(os.path.join(
                     script_dir, CHECK_SCRIPT_NAME))
-            except Exception:
+            except Exception as e:
                 raise exceptions.Conflict(
                     description='%(file_name)s not Found for '
                                 'UDP Listener %(listener_id)s' %
                                 {'file_name': CHECK_SCRIPT_NAME,
-                                 'listener_id': listener_id})
+                                 'listener_id': listener_id}) from e
             os.makedirs(util.keepalived_backend_check_script_dir())
             shutil.copy2(os.path.join(script_dir, CHECK_SCRIPT_NAME),
                          util.keepalived_backend_check_script_path())
@@ -147,7 +147,7 @@ class KeepalivedLvs(udp_listener_base.UdpListenerApiServerBase):
             except subprocess.CalledProcessError as e:
                 LOG.debug('Failed to enable '
                           'octavia-keepalivedlvs service: '
-                          '%(err)s', {'err': str(e)})
+                          '%(err)s', {'err': e})
                 return webob.Response(json=dict(
                     message="Error enabling "
                             "octavia-keepalivedlvs service",
@@ -223,7 +223,7 @@ class KeepalivedLvs(udp_listener_base.UdpListenerApiServerBase):
             subprocess.check_output(cmd.split(), stderr=subprocess.STDOUT)
         except subprocess.CalledProcessError as e:
             LOG.debug('Failed to %s keepalivedlvs listener %s',
-                      listener_id + ' : ' + action, str(e))
+                      listener_id + ' : ' + action, e)
             return webob.Response(json=dict(
                 message=("Failed to {0} keepalivedlvs listener {1}"
                          .format(action, listener_id)),
@@ -284,7 +284,7 @@ class KeepalivedLvs(udp_listener_base.UdpListenerApiServerBase):
             try:
                 subprocess.check_output(cmd.split(), stderr=subprocess.STDOUT)
             except subprocess.CalledProcessError as e:
-                LOG.error("Failed to stop keepalivedlvs service: %s", str(e))
+                LOG.error("Failed to stop keepalivedlvs service: %s", e)
                 return webob.Response(json=dict(
                     message="Error stopping keepalivedlvs",
                     details=e.output), status=500)
@@ -317,7 +317,7 @@ class KeepalivedLvs(udp_listener_base.UdpListenerApiServerBase):
             except subprocess.CalledProcessError as e:
                 LOG.error("Failed to disable "
                           "octavia-keepalivedlvs-%(list)s service: "
-                          "%(err)s", {'list': listener_id, 'err': str(e)})
+                          "%(err)s", {'list': listener_id, 'err': e})
                 return webob.Response(json=dict(
                     message=(
                         "Error disabling octavia-keepalivedlvs-"
