@@ -17,10 +17,12 @@
 import errno
 import os
 import queue
+import stat
 import time
 
 from oslo_config import cfg
 from oslo_log import log as logging
+import simplejson
 
 from octavia.amphorae.backends.agent.api_server import util
 from octavia.amphorae.backends.health_daemon import health_sender
@@ -38,8 +40,57 @@ SEQ = 0
 #
 # ver 1 - Adds UDP listener status when no pool or members are present
 # ver 2 - Switch to all listeners in a single combined haproxy config
-#
-MSG_VER = 2
+# ver 3 - Switch stats reporting to deltas
+
+MSG_VER = 3
+
+DELTA_METRICS = ('bin', 'bout', 'ereq', 'stot')
+
+# Filesystem persistent counters for statistics deltas
+COUNTERS = None
+COUNTERS_FILE = None
+
+
+def get_counters_file():
+    global COUNTERS_FILE
+    if COUNTERS_FILE is None:
+        stats_file_path = os.path.join(
+            CONF.haproxy_amphora.base_path, "stats_counters.json")
+        # Open for read+write and create if necessary
+        flags = os.O_RDWR | os.O_CREAT
+        # mode 00644
+        mode = stat.S_IRUSR | stat.S_IWUSR | stat.S_IRGRP
+        try:
+            COUNTERS_FILE = os.fdopen(
+                os.open(stats_file_path, flags, mode), 'r+')
+        except OSError:
+            LOG.info("Failed to open `%s`, ignoring...", stats_file_path)
+    COUNTERS_FILE.seek(0)
+    return COUNTERS_FILE
+
+
+def get_counters():
+    global COUNTERS
+    if COUNTERS is None:
+        try:
+            COUNTERS = simplejson.load(get_counters_file()) or {}
+        except (simplejson.JSONDecodeError, AttributeError):
+            COUNTERS = {}
+    return COUNTERS
+
+
+def persist_counters():
+    """Attempt to persist the latest statistics values"""
+    if COUNTERS is None:
+        return
+    try:
+        stats = simplejson.dumps(COUNTERS)
+        counters_file = get_counters_file()
+        counters_file.truncate(0)
+        counters_file.write(stats)
+        counters_file.flush()
+    except (OSError, AttributeError):
+        LOG.warning("Couldn't persist statistics counter file!")
 
 
 def list_sock_stat_files(hadir=None):
@@ -83,12 +134,12 @@ def run_sender(cmd_queue):
                 LOG.error('Keepalived is configured but not running, '
                           'skipping health heartbeat.')
             else:
-                LOG.error('Failed to check keepalived and haproxy status due '
-                          'to exception %s, skipping health heartbeat.',
-                          str(e))
+                LOG.exception('Failed to check keepalived and haproxy status '
+                              'due to exception %s, skipping health '
+                              'heartbeat.', e)
         except Exception as e:
-            LOG.error('Failed to check keepalived and haproxy status due to '
-                      'exception %s, skipping health heartbeat.', str(e))
+            LOG.exception('Failed to check keepalived and haproxy status due '
+                          'to exception %s, skipping health heartbeat.', e)
 
         try:
             cmd = cmd_queue.get_nowait()
@@ -110,33 +161,59 @@ def get_stats(stat_sock_file):
     return stats, pool_status
 
 
+def calculate_stats_deltas(listener_id, row):
+    counters = get_counters()
+    listener_counters = counters.get(listener_id, {})
+    counters[listener_id] = listener_counters
+
+    delta_values = {}
+    for metric_key in DELTA_METRICS:
+        current_value = int(row[metric_key])
+        # Get existing counter for our metrics
+        last_value = listener_counters.get(metric_key, 0)
+        # Store the new absolute value
+        listener_counters[metric_key] = current_value
+        # Calculate a delta for each metric
+        delta = current_value - last_value
+        # Did HAProxy restart or reset counters?
+        if delta < 0:
+            delta = current_value  # If so, reset ours.
+        delta_values[metric_key] = delta
+
+    return delta_values
+
+
 def build_stats_message():
-    # Example version 2 message without UDP:
-    # {
-    #   "id": "<amphora_id>",
-    #   "seq": 67,
-    #   "listeners": {
-    #     "<listener_id>": {
-    #       "status": "OPEN",
-    #       "stats": {
-    #         "tx": 0,
-    #         "rx": 0,
-    #         "conns": 0,
-    #         "totconns": 0,
-    #         "ereq": 0
-    #       }
-    #     }
-    #  },
-    #  "pools": {
-    #    "<pool_id>:<listener_id>": {
-    #      "status": "UP",
-    #      "members": {
-    #        "<member_id>": "no check"
-    #      }
-    #    }
-    #  },
-    #  "ver": 2
-    # }
+    """Build a stats message based on retrieved listener statistics.
+
+    Example version 3 message without UDP (note that values are deltas,
+    not absolutes)::
+
+        {"id": "<amphora_id>",
+         "seq": 67,
+         "listeners": {
+           "<listener_id>": {
+             "status": "OPEN",
+             "stats": {
+               "tx": 0,
+               "rx": 0,
+               "conns": 0,
+               "totconns": 0,
+               "ereq": 0
+             }
+           }
+         },
+         "pools": {
+             "<pool_id>:<listener_id>": {
+               "status": "UP",
+               "members": {
+                 "<member_id>": "no check"
+               }
+             }
+         },
+         "ver": 3
+        }
+    """
     global SEQ
     msg = {'id': CONF.amphora_agent.amphora_id,
            'seq': SEQ, 'listeners': {}, 'pools': {},
@@ -150,13 +227,14 @@ def build_stats_message():
             for row in stats:
                 if row['svname'] == 'FRONTEND':
                     listener_id = row['pxname']
+                    delta_values = calculate_stats_deltas(listener_id, row)
                     msg['listeners'][listener_id] = {
                         'status': row['status'],
-                        'stats': {'tx': int(row['bout']),
-                                  'rx': int(row['bin']),
+                        'stats': {'tx': delta_values['bout'],
+                                  'rx': delta_values['bin'],
                                   'conns': int(row['scur']),
-                                  'totconns': int(row['stot']),
-                                  'ereq': int(row['ereq'])}}
+                                  'totconns': delta_values['stot'],
+                                  'ereq': delta_values['ereq']}}
             for pool_id, pool in pool_status.items():
                 msg['pools'][pool_id] = {"status": pool['status'],
                                          "members": pool['members']}
@@ -167,16 +245,18 @@ def build_stats_message():
         listeners_stats = keepalivedlvs_query.get_udp_listeners_stats()
         if listeners_stats:
             for listener_id, listener_stats in listeners_stats.items():
+                delta_values = calculate_stats_deltas(
+                    listener_id, listener_stats['stats'])
                 pool_status = keepalivedlvs_query.get_udp_listener_pool_status(
                     listener_id)
                 udp_listener_dict = dict()
                 udp_listener_dict['status'] = listener_stats['status']
                 udp_listener_dict['stats'] = {
-                    'tx': listener_stats['stats']['bout'],
-                    'rx': listener_stats['stats']['bin'],
+                    'tx': delta_values['bout'],
+                    'rx': delta_values['bin'],
                     'conns': listener_stats['stats']['scur'],
-                    'totconns': listener_stats['stats']['stot'],
-                    'ereq': listener_stats['stats']['ereq']
+                    'totconns': delta_values['stot'],
+                    'ereq': delta_values['ereq']
                 }
                 if pool_status:
                     pool_id = pool_status['lvs']['uuid']
@@ -185,4 +265,5 @@ def build_stats_message():
                         "members": pool_status['lvs']['members']
                     }
                 msg['listeners'][listener_id] = udp_listener_dict
+    persist_counters()
     return msg
