@@ -18,6 +18,7 @@ from dateutil import parser
 import netaddr
 from wsme import types as wtypes
 
+from octavia.common import constants
 from octavia.common import exceptions
 from octavia.common import validate
 
@@ -36,9 +37,9 @@ class IPAddressType(wtypes.UserType):
             try:
                 wtypes.IPv6AddressType.validate(value)
                 return value
-            except ValueError:
+            except ValueError as e:
                 error = 'Value should be IPv4 or IPv6 format'
-                raise ValueError(error)
+                raise ValueError(error) from e
 
 
 class CidrType(wtypes.UserType):
@@ -50,9 +51,22 @@ class CidrType(wtypes.UserType):
         """Validates whether value is an IPv4 or IPv6 CIDR."""
         try:
             return str(netaddr.IPNetwork(value).cidr)
-        except (ValueError, netaddr.core.AddrFormatError):
+        except (ValueError, netaddr.core.AddrFormatError) as e:
             error = 'Value should be IPv4 or IPv6 CIDR format'
-            raise ValueError(error)
+            raise ValueError(error) from e
+
+
+class AlpnProtocolType(wtypes.UserType):
+    basetype = str
+    name = 'alpn_protocol'
+
+    @staticmethod
+    def validate(value):
+        """Validates whether value is a valid ALPN protocol ID."""
+        if value in constants.SUPPORTED_ALPN_PROTOCOLS:
+            return value
+        error = 'Value should be a valid ALPN protocol ID'
+        raise ValueError(error)
 
 
 class URLType(wtypes.UserType):
@@ -60,15 +74,15 @@ class URLType(wtypes.UserType):
     name = 'url'
 
     def __init__(self, require_scheme=True):
-        super(URLType, self).__init__()
+        super().__init__()
         self.require_scheme = require_scheme
 
     def validate(self, value):
         try:
             validate.url(value, require_scheme=self.require_scheme)
-        except exceptions.InvalidURL:
+        except exceptions.InvalidURL as e:
             error = 'Value must be a valid URL string'
-            raise ValueError(error)
+            raise ValueError(error) from e
         return value
 
 
@@ -80,9 +94,9 @@ class URLPathType(wtypes.UserType):
     def validate(value):
         try:
             validate.url_path(value)
-        except exceptions.InvalidURLPath:
+        except exceptions.InvalidURLPath as e:
             error = 'Value must be a valid URL Path string'
-            raise ValueError(error)
+            raise ValueError(error) from e
         return value
 
 
