@@ -38,7 +38,7 @@ class BarbicanCertManager(cert_mgr.CertManager):
     """Certificate Manager that wraps the Barbican client API."""
 
     def __init__(self):
-        super(BarbicanCertManager, self).__init__()
+        super().__init__()
         self.auth = stevedore_driver.DriverManager(
             namespace='octavia.barbican_auth',
             name=cfg.CONF.certificates.barbican_auth,
@@ -90,7 +90,7 @@ class BarbicanCertManager(cert_mgr.CertManager):
             return certificate_secret.secret_ref
         except Exception as e:
             with excutils.save_and_reraise_exception():
-                LOG.error('Error storing certificate data: %s', str(e))
+                LOG.error('Error storing certificate data: %s', e)
 
     def get_cert(self, context, cert_ref, resource_ref=None, check_only=False,
                  service_name=None):
