@@ -31,6 +31,7 @@ import oslo_messaging as messaging
 from octavia.certificates.common import local
 from octavia.common import constants
 from octavia.common import utils
+from octavia.common import validate
 from octavia.i18n import _
 from octavia import version
 
@@ -104,9 +105,6 @@ api_opts = [
     cfg.BoolOpt('healthcheck_enabled', default=False,
                 help=_("When True, the oslo middleware healthcheck endpoint "
                        "is enabled in the Octavia API.")),
-    cfg.IntOpt('healthcheck_refresh_interval', default=5,
-               help=_("The interval healthcheck plugins should cache results, "
-                      "in seconds.")),
     cfg.StrOpt('default_listener_ciphers',
                default=constants.CIPHERS_OWASP_SUITE_B,
                help=_("Default OpenSSL cipher string (colon-separated) for "
@@ -115,6 +113,27 @@ api_opts = [
                default=constants.CIPHERS_OWASP_SUITE_B,
                help=_("Default OpenSSL cipher string (colon-separated) for "
                       "new TLS-enabled pools.")),
+    cfg.StrOpt('tls_cipher_prohibit_list', default='',
+               deprecated_name='tls_cipher_blacklist',
+               help=_("Colon separated list of OpenSSL ciphers. "
+                      "Usage of these ciphers will be blocked.")),
+    cfg.ListOpt('default_listener_tls_versions',
+                default=constants.TLS_VERSIONS_OWASP_SUITE_B,
+                help=_('List of TLS versions to use for new TLS-enabled '
+                       'listeners.')),
+    cfg.ListOpt('default_pool_tls_versions',
+                default=constants.TLS_VERSIONS_OWASP_SUITE_B,
+                help=_('List of TLS versions to use for new TLS-enabled '
+                       'pools.')),
+    cfg.StrOpt('minimum_tls_version',
+               default=None,
+               choices=constants.TLS_ALL_VERSIONS + [None],
+               help=_('Minimum allowed TLS version for listeners and pools.')),
+    cfg.ListOpt('default_listener_alpn_protocols',
+                default=[lib_consts.ALPN_PROTOCOL_HTTP_1_1,
+                         lib_consts.ALPN_PROTOCOL_HTTP_1_0],
+                help=_('List of ALPN protocols to use for new TLS-enabled '
+                       'listeners.')),
 ]
 
 # Options only used by the amphora agent
@@ -242,7 +261,7 @@ networking_opts = [
                        "neutron RBAC policies.")),
 ]
 
-healthmanager_opts = [
+health_manager_opts = [
     cfg.IPOpt('bind_ip', default='127.0.0.1',
               help=_('IP address the controller will listen on for '
                      'heart beats')),
@@ -252,14 +271,6 @@ healthmanager_opts = [
     cfg.IntOpt('failover_threads',
                default=10,
                help=_('Number of threads performing amphora failovers.')),
-    # TODO(tatsuma) Remove in or after "T" release
-    cfg.IntOpt('status_update_threads',
-               default=None,
-               help=_('Number of processes for amphora status update.'),
-               deprecated_for_removal=True,
-               deprecated_reason=_('This option is replaced as '
-                                   'health_update_threads and '
-                                   'stats_update_threads')),
     cfg.IntOpt('health_update_threads',
                default=None,
                help=_('Number of processes for amphora health update.')),
@@ -292,11 +303,12 @@ healthmanager_opts = [
                mutable=True,
                help=_('Sleep time between sending heartbeats.')),
 
-    # Used for updating health and stats
+    # Used for updating health
     cfg.StrOpt('health_update_driver', default='health_db',
-               help=_('Driver for updating amphora health system.')),
-    cfg.StrOpt('stats_update_driver', default='stats_db',
-               help=_('Driver for updating amphora statistics.')),
+               help=_('Driver for updating amphora health system.'),
+               deprecated_for_removal=True,
+               deprecated_reason=_('This driver interface was removed.'),
+               deprecated_since='Victoria'),
 ]
 
 oslo_messaging_opts = [
@@ -434,13 +446,7 @@ controller_worker_opts = [
                default='',
                help=_('Glance image tag for the Amphora image to boot. '
                       'Use this option to be able to update the image '
-                      'without reconfiguring Octavia. '
-                      'Ignored if amp_image_id is defined.')),
-    cfg.StrOpt('amp_image_id',
-               default='',
-               deprecated_for_removal=True,
-               deprecated_reason='Superseded by amp_image_tag option.',
-               help=_('Glance image id for the Amphora image to boot')),
+                      'without reconfiguring Octavia.')),
     cfg.StrOpt('amp_image_owner_id',
                default='',
                help=_('Restrict glance image selection to a specific '
@@ -449,15 +455,6 @@ controller_worker_opts = [
                default='',
                help=_('Optional SSH keypair name, in nova, that will be used '
                       'for the authorized_keys inside the amphora.')),
-    cfg.BoolOpt('amp_ssh_access_allowed',
-                default=True,
-                deprecated_for_removal=True,
-                deprecated_reason='This option and amp_ssh_key_name overlap '
-                                  'in functionality, and only one is needed. '
-                                  'SSH access can be enabled/disabled simply '
-                                  'by setting amp_ssh_key_name, or not.',
-                help=_('Determines whether or not to allow access '
-                       'to the Amphorae')),
     cfg.ListOpt('amp_boot_network_list',
                 default='',
                 help=_('List of networks to attach to the Amphorae. '
@@ -470,21 +467,30 @@ controller_worker_opts = [
                default='/etc/octavia/certs/ca_01.pem',
                help=_('Client CA for the amphora agent to use')),
     cfg.StrOpt('amphora_driver',
-               default='amphora_noop_driver',
+               default='amphora_haproxy_rest_driver',
                help=_('Name of the amphora driver to use')),
     cfg.StrOpt('compute_driver',
-               default='compute_noop_driver',
+               default='compute_nova_driver',
                help=_('Name of the compute driver to use')),
     cfg.StrOpt('network_driver',
-               default='network_noop_driver',
+               default='allowed_address_pairs_driver',
                help=_('Name of the network driver to use')),
     cfg.StrOpt('volume_driver',
                default=constants.VOLUME_NOOP_DRIVER,
                choices=constants.SUPPORTED_VOLUME_DRIVERS,
                help=_('Name of the volume driver to use')),
+    cfg.StrOpt('image_driver',
+               default='image_glance_driver',
+               choices=constants.SUPPORTED_IMAGE_DRIVERS,
+               help=_('Name of the image driver to use')),
     cfg.StrOpt('distributor_driver',
                default='distributor_noop_driver',
                help=_('Name of the distributor driver to use')),
+    cfg.ListOpt('statistics_drivers', default=['stats_db'],
+                deprecated_name='stats_update_driver',
+                deprecated_group='health_manager',
+                deprecated_since='Victoria',
+                help=_('List of drivers for updating amphora statistics.')),
     cfg.StrOpt('loadbalancer_topology',
                default=constants.TOPOLOGY_SINGLE,
                choices=constants.SUPPORTED_LB_TOPOLOGIES,
@@ -523,6 +529,8 @@ task_flow_opts = [
                default='sqlite://',
                help='Persistence database, which will be used to store tasks '
                     'states. Database connection url with db name'),
+    cfg.BoolOpt('jobboard_enabled', default=False,
+                help=_('If True, enables TaskFlow jobboard.')),
     cfg.StrOpt('jobboard_backend_driver',
                default='redis_taskflow_driver',
                choices=['redis_taskflow_driver', 'zookeeper_taskflow_driver'],
@@ -536,6 +544,8 @@ task_flow_opts = [
     cfg.StrOpt('jobboard_backend_namespace', default='octavia_jobboard',
                help='Jobboard name that should be used to store taskflow '
                     'job id and claims for it.'),
+    cfg.StrOpt('jobboard_redis_sentinel', default=None,
+               help='Sentinel name if it is used for Redis.'),
     cfg.DictOpt('jobboard_redis_backend_ssl_options',
                 help='Redis jobboard backend ssl configuration options.',
                 default={'ssl': False,
@@ -584,8 +594,7 @@ certificate_opts = [
                default='publicURL',
                help='The endpoint_type to be used for barbican service.'),
     cfg.StrOpt('ca_certificates_file',
-               help=_('CA certificates file path for the key manager service '
-                      '(such as Barbican).')),
+               help=_('CA certificates file path')),
     cfg.BoolOpt('insecure',
                 default=False,
                 help=_('Disable certificate validation on SSL connections ')),
@@ -593,9 +602,17 @@ certificate_opts = [
 
 house_keeping_opts = [
     cfg.IntOpt('spare_check_interval',
+               deprecated_for_removal=True,
+               deprecated_since='Victoria',
+               deprecated_reason='Spares Pool support will be removed in the '
+                                 'X release.',
                default=30,
                help=_('Spare check interval in seconds')),
     cfg.IntOpt('spare_amphora_pool_size',
+               deprecated_for_removal=True,
+               deprecated_since='Victoria',
+               deprecated_reason='Spares Pool support will be removed in the '
+                                 'X release.',
                default=0,
                help=_('Number of spare amphorae')),
     cfg.IntOpt('cleanup_interval',
@@ -761,6 +778,12 @@ quota_opts = [
     cfg.IntOpt('default_health_monitor_quota',
                default=constants.QUOTA_UNLIMITED,
                help=_('Default per project health monitor quota.')),
+    cfg.IntOpt('default_l7policy_quota',
+               default=constants.QUOTA_UNLIMITED,
+               help=_('Default per project l7policy quota.')),
+    cfg.IntOpt('default_l7rule_quota',
+               default=constants.QUOTA_UNLIMITED,
+               help=_('Default per project l7rule quota.')),
 ]
 
 audit_opts = [
@@ -839,7 +862,7 @@ cfg.CONF.register_opts(keepalived_vrrp_opts, group='keepalived_vrrp')
 cfg.CONF.register_opts(task_flow_opts, group='task_flow')
 cfg.CONF.register_opts(house_keeping_opts, group='house_keeping')
 cfg.CONF.register_opts(certificate_opts, group='certificates')
-cfg.CONF.register_opts(healthmanager_opts, group='health_manager')
+cfg.CONF.register_opts(health_manager_opts, group='health_manager')
 cfg.CONF.register_opts(nova_opts, group='nova')
 cfg.CONF.register_opts(cinder_opts, group='cinder')
 cfg.CONF.register_opts(glance_opts, group='glance')
@@ -873,8 +896,9 @@ def init(args, **kwargs):
     cfg.CONF(args=args, project='octavia',
              version='%%prog %s' % version.version_info.release_string(),
              **kwargs)
-    handle_deprecation_compatibility()
+    validate.check_default_tls_versions_min_conflict()
     setup_remote_debugger()
+    validate.check_default_ciphers_prohibit_list_conflict()
 
 
 def setup_logging(conf):
@@ -893,21 +917,6 @@ def setup_logging(conf):
     LOG.debug("command line: %s", " ".join(sys.argv))
 
 
-# Use cfg.CONF.set_default to override the new configuration setting
-# default value.  This allows a value set, at the new location, to override
-# a value set in the previous location while allowing settings that have
-# not yet been moved to be utilized.
-def handle_deprecation_compatibility():
-    # TODO(tatsuma) Remove in or after "T" release
-    if cfg.CONF.health_manager.status_update_threads is not None:
-        cfg.CONF.set_default('health_update_threads',
-                             cfg.CONF.health_manager.status_update_threads,
-                             group='health_manager')
-        cfg.CONF.set_default('stats_update_threads',
-                             cfg.CONF.health_manager.status_update_threads,
-                             group='health_manager')
-
-
 def _enable_pydev(debugger_host, debugger_port):
     try:
         from pydev import pydevd  # pylint: disable=import-outside-toplevel
