@@ -17,13 +17,10 @@
 import datetime
 import re
 
-from oslo_log import log as logging
 from sqlalchemy.orm import collections
 
 from octavia.common import constants
 
-LOG = logging.getLogger(__name__)
-
 
 class BaseDataModel(object):
     def to_dict(self, calling_classes=None, recurse=False, **kwargs):
@@ -43,9 +40,6 @@ class BaseDataModel(object):
             if isinstance(value, datetime.datetime):
                 ret[attr] = value.isoformat()
                 continue
-            if isinstance(value, bytes):
-                ret[attr] = value.decode()
-                continue
             if recurse:
                 if isinstance(getattr(self, attr), list):
                     ret[attr] = []
@@ -188,7 +182,7 @@ class ListenerStatistics(BaseDataModel):
 
     def __init__(self, listener_id=None, amphora_id=None, bytes_in=0,
                  bytes_out=0, active_connections=0,
-                 total_connections=0, request_errors=0):
+                 total_connections=0, request_errors=0, received_time=0.0):
         self.listener_id = listener_id
         self.amphora_id = amphora_id
         self.bytes_in = bytes_in
@@ -196,6 +190,7 @@ class ListenerStatistics(BaseDataModel):
         self.active_connections = active_connections
         self.total_connections = total_connections
         self.request_errors = request_errors
+        self.received_time = received_time
 
     def get_stats(self):
         stats = {
@@ -207,13 +202,21 @@ class ListenerStatistics(BaseDataModel):
         }
         return stats
 
-    def __iadd__(self, other):
+    def db_fields(self):
+        fields = self.to_dict()
+        fields.pop('received_time')
+        return fields
 
+    def __iadd__(self, other):
         if isinstance(other, ListenerStatistics):
             self.bytes_in += other.bytes_in
             self.bytes_out += other.bytes_out
             self.request_errors += other.request_errors
             self.total_connections += other.total_connections
+        else:
+            raise TypeError(  # noqa: O342
+                "unsupported operand type(s) for +=: '{0}' and '{1}'".format(
+                    type(self), type(other)))
 
         return self
 
@@ -284,7 +287,7 @@ class Pool(BaseDataModel):
                  created_at=None, updated_at=None, provisioning_status=None,
                  tags=None, tls_certificate_id=None,
                  ca_tls_certificate_id=None, crl_container_id=None,
-                 tls_enabled=None, tls_ciphers=None):
+                 tls_enabled=None, tls_ciphers=None, tls_versions=None):
         self.id = id
         self.project_id = project_id
         self.name = name
@@ -309,6 +312,7 @@ class Pool(BaseDataModel):
         self.crl_container_id = crl_container_id
         self.tls_enabled = tls_enabled
         self.tls_ciphers = tls_ciphers
+        self.tls_versions = tls_versions
 
     def update(self, update_dict):
         for key, value in update_dict.items():
@@ -335,11 +339,7 @@ class Pool(BaseDataModel):
                     break
         for pool in self.load_balancer.pools:
             if pool.id == self.id:
-                try:
-                    self.load_balancer.pools.remove(pool)
-                except ValueError:
-                    LOG.debug("Pool %s has already been removed from load "
-                              "balancer pools list.", pool.id)
+                self.load_balancer.pools.remove(pool)
                 break
         for l7policy in self.l7policies:
             if l7policy.redirect_pool_id == self.id:
@@ -400,7 +400,8 @@ class Listener(BaseDataModel):
                  timeout_member_data=None, timeout_tcp_inspect=None,
                  tags=None, client_ca_tls_certificate_id=None,
                  client_authentication=None, client_crl_container_id=None,
-                 allowed_cidrs=None, tls_ciphers=None):
+                 allowed_cidrs=None, tls_ciphers=None, tls_versions=None,
+                 alpn_protocols=None):
         self.id = id
         self.project_id = project_id
         self.name = name
@@ -434,6 +435,8 @@ class Listener(BaseDataModel):
         self.client_crl_container_id = client_crl_container_id
         self.allowed_cidrs = allowed_cidrs or []
         self.tls_ciphers = tls_ciphers
+        self.tls_versions = tls_versions
+        self.alpn_protocols = alpn_protocols
 
     def update(self, update_dict):
         for key, value in update_dict.items():
@@ -764,22 +767,30 @@ class Quotas(BaseDataModel):
                  pool=None,
                  health_monitor=None,
                  member=None,
+                 l7policy=None,
+                 l7rule=None,
                  in_use_health_monitor=None,
                  in_use_listener=None,
                  in_use_load_balancer=None,
                  in_use_member=None,
-                 in_use_pool=None):
+                 in_use_pool=None,
+                 in_use_l7policy=None,
+                 in_use_l7rule=None):
         self.project_id = project_id
         self.health_monitor = health_monitor
         self.listener = listener
         self.load_balancer = load_balancer
         self.pool = pool
         self.member = member
+        self.l7policy = l7policy
+        self.l7rule = l7rule
         self.in_use_health_monitor = in_use_health_monitor
         self.in_use_listener = in_use_listener
         self.in_use_load_balancer = in_use_load_balancer
         self.in_use_member = in_use_member
         self.in_use_pool = in_use_pool
+        self.in_use_l7policy = in_use_l7policy
+        self.in_use_l7rule = in_use_l7rule
 
 
 class Flavor(BaseDataModel):
