@@ -36,7 +36,7 @@ class TaskUtils(object):
         self.amp_health_repo = repo.AmphoraHealthRepository()
         self.l7policy_repo = repo.L7PolicyRepository()
         self.l7rule_repo = repo.L7RuleRepository()
-        super(TaskUtils, self).__init__(**kwargs)
+        super().__init__(**kwargs)
 
     def unmark_amphora_health_busy(self, amphora_id):
         """Unmark the amphora_health record busy for an amphora.
@@ -53,8 +53,7 @@ class TaskUtils(object):
                                         busy=False)
         except Exception as e:
             LOG.debug('Failed to update amphora health record %(amp)s '
-                      'due to: %(except)s',
-                      {'amp': amphora_id, 'except': str(e)})
+                      'due to: %(except)s', {'amp': amphora_id, 'except': e})
 
     def mark_amphora_status_error(self, amphora_id):
         """Sets an amphora status to ERROR.
@@ -70,7 +69,7 @@ class TaskUtils(object):
         except Exception as e:
             LOG.error("Failed to update amphora %(amp)s "
                       "status to ERROR due to: "
-                      "%(except)s", {'amp': amphora_id, 'except': str(e)})
+                      "%(except)s", {'amp': amphora_id, 'except': e})
 
     def mark_health_mon_prov_status_error(self, health_mon_id):
         """Sets a health monitor provisioning status to ERROR.
@@ -86,8 +85,7 @@ class TaskUtils(object):
         except Exception as e:
             LOG.error("Failed to update health monitor %(health)s "
                       "provisioning status to ERROR due to: "
-                      "%(except)s",
-                      {'health': health_mon_id, 'except': str(e)})
+                      "%(except)s", {'health': health_mon_id, 'except': e})
 
     def mark_l7policy_prov_status_active(self, l7policy_id):
         """Sets a L7 policy provisioning status to ACTIVE.
@@ -103,7 +101,7 @@ class TaskUtils(object):
         except Exception as e:
             LOG.error("Failed to update l7policy %(l7p)s "
                       "provisioning status to ACTIVE due to: "
-                      "%(except)s", {'l7p': l7policy_id, 'except': str(e)})
+                      "%(except)s", {'l7p': l7policy_id, 'except': e})
 
     def mark_l7policy_prov_status_error(self, l7policy_id):
         """Sets a L7 policy provisioning status to ERROR.
@@ -119,7 +117,7 @@ class TaskUtils(object):
         except Exception as e:
             LOG.error("Failed to update l7policy %(l7p)s "
                       "provisioning status to ERROR due to: "
-                      "%(except)s", {'l7p': l7policy_id, 'except': str(e)})
+                      "%(except)s", {'l7p': l7policy_id, 'except': e})
 
     def mark_l7rule_prov_status_error(self, l7rule_id):
         """Sets a L7 rule provisioning status to ERROR.
@@ -135,7 +133,7 @@ class TaskUtils(object):
         except Exception as e:
             LOG.error("Failed to update l7rule %(l7r)s "
                       "provisioning status to ERROR due to: "
-                      "%(except)s", {'l7r': l7rule_id, 'except': str(e)})
+                      "%(except)s", {'l7r': l7rule_id, 'except': e})
 
     def mark_listener_prov_status_error(self, listener_id):
         """Sets a listener provisioning status to ERROR.
@@ -151,7 +149,7 @@ class TaskUtils(object):
         except Exception as e:
             LOG.error("Failed to update listener %(list)s "
                       "provisioning status to ERROR due to: "
-                      "%(except)s", {'list': listener_id, 'except': str(e)})
+                      "%(except)s", {'list': listener_id, 'except': e})
 
     def mark_loadbalancer_prov_status_error(self, loadbalancer_id):
         """Sets a load balancer provisioning status to ERROR.
@@ -168,7 +166,7 @@ class TaskUtils(object):
         except Exception as e:
             LOG.error("Failed to update load balancer %(lb)s "
                       "provisioning status to ERROR due to: "
-                      "%(except)s", {'lb': loadbalancer_id, 'except': str(e)})
+                      "%(except)s", {'lb': loadbalancer_id, 'except': e})
 
     def mark_listener_prov_status_active(self, listener_id):
         """Sets a listener provisioning status to ACTIVE.
@@ -185,7 +183,7 @@ class TaskUtils(object):
         except Exception as e:
             LOG.error("Failed to update listener %(list)s "
                       "provisioning status to ACTIVE due to: "
-                      "%(except)s", {'list': listener_id, 'except': str(e)})
+                      "%(except)s", {'list': listener_id, 'except': e})
 
     def mark_pool_prov_status_active(self, pool_id):
         """Sets a pool provisioning status to ACTIVE.
@@ -201,7 +199,7 @@ class TaskUtils(object):
         except Exception as e:
             LOG.error("Failed to update pool %(pool)s provisioning status "
                       "to ACTIVE due to: %(except)s", {'pool': pool_id,
-                                                       'except': str(e)})
+                                                       'except': e})
 
     def mark_loadbalancer_prov_status_active(self, loadbalancer_id):
         """Sets a load balancer provisioning status to ACTIVE.
@@ -218,7 +216,7 @@ class TaskUtils(object):
         except Exception as e:
             LOG.error("Failed to update load balancer %(lb)s "
                       "provisioning status to ACTIVE due to: "
-                      "%(except)s", {'lb': loadbalancer_id, 'except': str(e)})
+                      "%(except)s", {'lb': loadbalancer_id, 'except': e})
 
     def mark_member_prov_status_error(self, member_id):
         """Sets a member provisioning status to ERROR.
@@ -234,7 +232,7 @@ class TaskUtils(object):
         except Exception as e:
             LOG.error("Failed to update member %(member)s "
                       "provisioning status to ERROR due to: "
-                      "%(except)s", {'member': member_id, 'except': str(e)})
+                      "%(except)s", {'member': member_id, 'except': e})
 
     def mark_pool_prov_status_error(self, pool_id):
         """Sets a pool provisioning status to ERROR.
@@ -250,7 +248,7 @@ class TaskUtils(object):
         except Exception as e:
             LOG.error("Failed to update pool %(pool)s "
                       "provisioning status to ERROR due to: "
-                      "%(except)s", {'pool': pool_id, 'except': str(e)})
+                      "%(except)s", {'pool': pool_id, 'except': e})
 
     def get_current_loadbalancer_from_db(self, loadbalancer_id):
         """Gets a Loadbalancer from db.
@@ -263,4 +261,4 @@ class TaskUtils(object):
         except Exception as e:
             LOG.error("Failed to get loadbalancer %(loadbalancer)s "
                       "due to: %(except)s",
-                      {'loadbalancer': loadbalancer_id, 'except': str(e)})
+                      {'loadbalancer': loadbalancer_id, 'except': e})
