@@ -36,7 +36,7 @@ class BaseAmphoraTask(task.Task):
     """Base task to load drivers common to the tasks."""
 
     def __init__(self, **kwargs):
-        super(BaseAmphoraTask, self).__init__(**kwargs)
+        super().__init__(**kwargs)
         self.amphora_driver = stevedore_driver.DriverManager(
             namespace='octavia.amphora.drivers',
             name=CONF.controller_worker.amphora_driver,
@@ -56,9 +56,6 @@ class AmpListenersUpdate(BaseAmphoraTask):
         # in a failover flow with both amps failing. Skip it and let
         # health manager fix it.
         try:
-            # Make sure we have a fresh load balancer object
-            loadbalancer = self.loadbalancer_repo.get(db_apis.get_session(),
-                                                      id=loadbalancer.id)
             self.amphora_driver.update_amphora_listeners(
                 loadbalancer, amphora, timeout_dict)
         except Exception as e:
@@ -78,9 +75,6 @@ class AmphoraIndexListenerUpdate(BaseAmphoraTask):
         # in a failover flow with both amps failing. Skip it and let
         # health manager fix it.
         try:
-            # Make sure we have a fresh load balancer object
-            loadbalancer = self.loadbalancer_repo.get(db_apis.get_session(),
-                                                      id=loadbalancer.id)
             self.amphora_driver.update_amphora_listeners(
                 loadbalancer, amphorae[amphora_index], timeout_dict)
         except Exception as e:
@@ -435,7 +429,7 @@ class AmphoraComputeConnectivityWait(BaseAmphoraTask):
 
 
 class AmphoraConfigUpdate(BaseAmphoraTask):
-    """Task to push a new amphora agent configuration to the amphroa."""
+    """Task to push a new amphora agent configuration to the amphora."""
 
     def execute(self, amphora, flavor):
         # Extract any flavor based settings
@@ -449,11 +443,11 @@ class AmphoraConfigUpdate(BaseAmphoraTask):
         agent_cfg_tmpl = agent_jinja_cfg.AgentJinjaTemplater()
         agent_config = agent_cfg_tmpl.build_agent_config(amphora.id, topology)
 
-        # Push the new configuration to the amphroa
+        # Push the new configuration to the amphora
         try:
             self.amphora_driver.update_amphora_agent_config(amphora,
                                                             agent_config)
         except driver_except.AmpDriverNotImplementedError:
-            LOG.error('Amphora %s does not support agent configuration '
+            LOG.error('Amphora {} does not support agent configuration '
                       'update. Please update the amphora image for this '
-                      'amphora. Skipping.', amphora.id)
+                      'amphora. Skipping.'.format(amphora.id))
