@@ -39,7 +39,7 @@ class BaseComputeTask(task.Task):
     """Base task to load drivers common to the tasks."""
 
     def __init__(self, **kwargs):
-        super(BaseComputeTask, self).__init__(**kwargs)
+        super().__init__(**kwargs)
         self.compute = stevedore_driver.DriverManager(
             namespace='octavia.compute.drivers',
             name=CONF.controller_worker.compute_driver,
@@ -73,9 +73,12 @@ class ComputeCreate(BaseComputeTask):
                                   CONF.controller_worker.loadbalancer_topology)
             amp_compute_flavor = flavor.get(
                 constants.COMPUTE_FLAVOR, CONF.controller_worker.amp_flavor_id)
+            amp_image_tag = flavor.get(
+                constants.AMP_IMAGE_TAG, CONF.controller_worker.amp_image_tag)
         else:
             topology = CONF.controller_worker.loadbalancer_topology
             amp_compute_flavor = CONF.controller_worker.amp_flavor_id
+            amp_image_tag = CONF.controller_worker.amp_image_tag
 
         if availability_zone:
             amp_availability_zone = availability_zone.get(
@@ -108,8 +111,7 @@ class ComputeCreate(BaseComputeTask):
             compute_id = self.compute.build(
                 name="amphora-" + amphora_id,
                 amphora_flavor=amp_compute_flavor,
-                image_id=CONF.controller_worker.amp_image_id,
-                image_tag=CONF.controller_worker.amp_image_tag,
+                image_tag=amp_image_tag,
                 image_owner=CONF.controller_worker.amp_image_owner_id,
                 key_name=key_name,
                 sec_groups=CONF.controller_worker.amp_secgroup_list,
@@ -164,7 +166,7 @@ class CertComputeCreate(ComputeCreate):
         config_drive_files = {
             '/etc/octavia/certs/server.pem': fer.decrypt(server_pem),
             '/etc/octavia/certs/client_ca.pem': ca}
-        return super(CertComputeCreate, self).execute(
+        return super().execute(
             amphora_id, config_drive_files=config_drive_files,
             build_type_priority=build_type_priority,
             server_group_id=server_group_id, ports=ports, flavor=flavor,
@@ -292,7 +294,7 @@ class NovaServerGroupCreate(BaseComputeTask):
             LOG.error("Failed to delete server group.  Resources may "
                       "still be in use for server group: %(sg)s due to "
                       "error: %(except)s",
-                      {'sg': server_group_id, 'except': str(e)})
+                      {'sg': server_group_id, 'except': e})
 
 
 class NovaServerGroupDelete(BaseComputeTask):
