@@ -19,8 +19,10 @@ from oslo_log import log as logging
 from oslo_utils import excutils
 from sqlalchemy.orm import exc as db_exceptions
 from stevedore import driver as stevedore_driver
+from taskflow.listeners import logging as tf_logging
 import tenacity
 
+from octavia.amphorae.driver_exceptions import exceptions as driver_exc
 from octavia.api.drivers import utils as provider_utils
 from octavia.common import base_taskflow
 from octavia.common import constants
@@ -35,6 +37,18 @@ CONF = cfg.CONF
 LOG = logging.getLogger(__name__)
 
 
+# We do not need to log retry exception information. Warning "Could not connect
+#  to instance" will be logged as usual.
+def retryMaskFilter(record):
+    if record.exc_info is not None and isinstance(
+            record.exc_info[1], driver_exc.AmpConnectionRetry):
+        return False
+    return True
+
+
+LOG.logger.addFilter(retryMaskFilter)
+
+
 def _is_provisioning_status_pending_update(lb_obj):
     return not lb_obj.provisioning_status == constants.PENDING_UPDATE
 
@@ -55,13 +69,16 @@ class ControllerWorker(object):
         self._flavor_repo = repo.FlavorRepository()
         self._az_repo = repo.AvailabilityZoneRepository()
 
-        persistence = tsk_driver.MysqlPersistenceDriver()
+        if CONF.task_flow.jobboard_enabled:
+            persistence = tsk_driver.MysqlPersistenceDriver()
 
-        self.jobboard_driver = stevedore_driver.DriverManager(
-            namespace='octavia.worker.jobboard_driver',
-            name=CONF.task_flow.jobboard_backend_driver,
-            invoke_args=(persistence,),
-            invoke_on_load=True).driver
+            self.jobboard_driver = stevedore_driver.DriverManager(
+                namespace='octavia.worker.jobboard_driver',
+                name=CONF.task_flow.jobboard_backend_driver,
+                invoke_args=(persistence,),
+                invoke_on_load=True).driver
+        else:
+            self.tf_engine = base_taskflow.BaseTaskFlowEngine()
 
     @tenacity.retry(
         retry=(
@@ -81,6 +98,15 @@ class ControllerWorker(object):
     def services_controller(self):
         return base_taskflow.TaskFlowServiceController(self.jobboard_driver)
 
+    def run_flow(self, func, *args, **kwargs):
+        if CONF.task_flow.jobboard_enabled:
+            self.services_controller.run_poster(func, *args, **kwargs)
+        else:
+            tf = self.tf_engine.taskflow_load(
+                func(*args), **kwargs)
+            with tf_logging.DynamicLoggingListener(tf, log=LOG):
+                tf.run()
+
     def create_amphora(self, availability_zone=None):
         """Creates an Amphora.
 
@@ -98,13 +124,11 @@ class ControllerWorker(object):
                 store[constants.AVAILABILITY_ZONE] = (
                     self._az_repo.get_availability_zone_metadata_dict(
                         db_apis.get_session(), availability_zone))
-            job_id = self.services_controller.run_poster(
+            self.run_flow(
                 flow_utils.get_create_amphora_flow,
                 store=store, wait=True)
-
-            return job_id
         except Exception as e:
-            LOG.error('Failed to create an amphora due to: %s', str(e))
+            LOG.error('Failed to create an amphora due to: {}'.format(str(e)))
 
     def delete_amphora(self, amphora_id):
         """Deletes an existing Amphora.
@@ -113,12 +137,18 @@ class ControllerWorker(object):
         :returns: None
         :raises AmphoraNotFound: The referenced Amphora was not found
         """
-        amphora = self._amphora_repo.get(db_apis.get_session(),
-                                         id=amphora_id)
-        store = {constants.AMPHORA: amphora.to_dict()}
-        self.services_controller.run_poster(
-            flow_utils.get_delete_amphora_flow,
-            store=store)
+        try:
+            amphora = self._amphora_repo.get(db_apis.get_session(),
+                                             id=amphora_id)
+            store = {constants.AMPHORA: amphora.to_dict()}
+            self.run_flow(
+                flow_utils.get_delete_amphora_flow,
+                store=store)
+        except Exception as e:
+            LOG.error('Failed to delete a amphora {0} due to: {1}'.format(
+                amphora_id, str(e)))
+            return
+        LOG.info('Finished deleting amphora %s.', amphora_id)
 
     @tenacity.retry(
         retry=tenacity.retry_if_exception_type(db_exceptions.NoResultFound),
@@ -142,7 +172,7 @@ class ControllerWorker(object):
         if not db_health_monitor:
             LOG.warning('Failed to fetch %s %s from DB. Retrying for up to '
                         '60 seconds.', 'healthmonitor',
-                        health_monitor[constants.HEALTHMONITOR_ID])
+                        db_health_monitor[constants.HEALTHMONITOR_ID])
             raise db_exceptions.NoResultFound
 
         pool = db_health_monitor.pool
@@ -160,7 +190,7 @@ class ControllerWorker(object):
                  constants.LISTENERS: listeners_dicts,
                  constants.LOADBALANCER_ID: load_balancer.id,
                  constants.LOADBALANCER: provider_lb}
-        self.services_controller.run_poster(
+        self.run_flow(
             flow_utils.get_create_health_monitor_flow,
             store=store)
 
@@ -190,7 +220,7 @@ class ControllerWorker(object):
                  constants.LOADBALANCER_ID: load_balancer.id,
                  constants.LOADBALANCER: provider_lb,
                  constants.PROJECT_ID: load_balancer.project_id}
-        self.services_controller.run_poster(
+        self.run_flow(
             flow_utils.get_delete_health_monitor_flow,
             store=store)
 
@@ -231,7 +261,7 @@ class ControllerWorker(object):
                  constants.LOADBALANCER_ID: load_balancer.id,
                  constants.LOADBALANCER: provider_lb,
                  constants.UPDATE_DICT: health_monitor_updates}
-        self.services_controller.run_poster(
+        self.run_flow(
             flow_utils.get_update_health_monitor_flow,
             store=store)
 
@@ -271,7 +301,7 @@ class ControllerWorker(object):
                  constants.LOADBALANCER: provider_lb,
                  constants.LOADBALANCER_ID: load_balancer.id}
 
-        self.services_controller.run_poster(
+        self.run_flow(
             flow_utils.get_create_listener_flow,
             store=store)
 
@@ -286,7 +316,7 @@ class ControllerWorker(object):
                  constants.LOADBALANCER_ID:
                      listener[constants.LOADBALANCER_ID],
                  constants.PROJECT_ID: listener[constants.PROJECT_ID]}
-        self.services_controller.run_poster(
+        self.run_flow(
             flow_utils.get_delete_listener_flow,
             store=store)
 
@@ -304,7 +334,7 @@ class ControllerWorker(object):
                  constants.UPDATE_DICT: listener_updates,
                  constants.LOADBALANCER_ID: db_lb.id,
                  constants.LISTENERS: [listener]}
-        self.services_controller.run_poster(
+        self.run_flow(
             flow_utils.get_update_listener_flow,
             store=store)
 
@@ -356,7 +386,7 @@ class ControllerWorker(object):
         store[constants.UPDATE_DICT] = {
             constants.TOPOLOGY: topology
         }
-        self.services_controller.run_poster(
+        self.run_flow(
             flow_utils.get_create_load_balancer_flow,
             topology, listeners=listeners_dicts,
             store=store)
@@ -377,11 +407,12 @@ class ControllerWorker(object):
         if cascade:
             listeners = flow_utils.get_listeners_on_lb(db_lb)
             pools = flow_utils.get_pools_on_lb(db_lb)
-            self.services_controller.run_poster(
+
+            self.run_flow(
                 flow_utils.get_cascade_delete_load_balancer_flow,
                 load_balancer, listeners, pools, store=store)
         else:
-            self.services_controller.run_poster(
+            self.run_flow(
                 flow_utils.get_delete_load_balancer_flow,
                 load_balancer, store=store)
 
@@ -399,18 +430,10 @@ class ControllerWorker(object):
                      original_load_balancer[constants.LOADBALANCER_ID],
                  constants.UPDATE_DICT: load_balancer_updates}
 
-        self.services_controller.run_poster(
+        self.run_flow(
             flow_utils.get_update_load_balancer_flow,
             store=store)
 
-    @tenacity.retry(
-        retry=tenacity.retry_if_exception_type(db_exceptions.NoResultFound),
-        wait=tenacity.wait_incrementing(
-            CONF.haproxy_amphora.api_db_commit_retry_initial_delay,
-            CONF.haproxy_amphora.api_db_commit_retry_backoff,
-            CONF.haproxy_amphora.api_db_commit_retry_max),
-        stop=tenacity.stop_after_attempt(
-            CONF.haproxy_amphora.api_db_commit_retry_attempts))
     def create_member(self, member):
         """Creates a pool member.
 
@@ -418,15 +441,8 @@ class ControllerWorker(object):
         :returns: None
         :raises NoSuitablePool: Unable to find the node pool
         """
-        db_member = self._member_repo.get(db_apis.get_session(),
-                                          id=member[constants.MEMBER_ID])
-        if not db_member:
-            LOG.warning('Failed to fetch %s %s from DB. Retrying for up to '
-                        '60 seconds.', 'l7member',
-                        member[constants.MEMBER_ID])
-            raise db_exceptions.NoResultFound
-
-        pool = db_member.pool
+        pool = self._pool_repo.get(db_apis.get_session(),
+                                   id=member[constants.POOL_ID])
         load_balancer = pool.load_balancer
         provider_lb = provider_utils.db_loadbalancer_to_provider_loadbalancer(
             load_balancer).to_dict()
@@ -448,7 +464,7 @@ class ControllerWorker(object):
         else:
             store[constants.AVAILABILITY_ZONE] = {}
 
-        self.services_controller.run_poster(
+        self.run_flow(
             flow_utils.get_create_member_flow,
             store=store)
 
@@ -484,7 +500,7 @@ class ControllerWorker(object):
         else:
             store[constants.AVAILABILITY_ZONE] = {}
 
-        self.services_controller.run_poster(
+        self.run_flow(
             flow_utils.get_delete_member_flow,
             store=store)
 
@@ -532,7 +548,7 @@ class ControllerWorker(object):
         else:
             store[constants.AVAILABILITY_ZONE] = {}
 
-        self.services_controller.run_poster(
+        self.run_flow(
             flow_utils.get_batch_update_members_flow,
             provider_old_members, new_members, updated_members,
             store=store)
@@ -569,7 +585,7 @@ class ControllerWorker(object):
         else:
             store[constants.AVAILABILITY_ZONE] = {}
 
-        self.services_controller.run_poster(
+        self.run_flow(
             flow_utils.get_update_member_flow,
             store=store)
 
@@ -610,7 +626,7 @@ class ControllerWorker(object):
                  constants.LISTENERS: listeners_dicts,
                  constants.LOADBALANCER_ID: load_balancer.id,
                  constants.LOADBALANCER: provider_lb}
-        self.services_controller.run_poster(
+        self.run_flow(
             flow_utils.get_create_pool_flow,
             store=store)
 
@@ -637,7 +653,7 @@ class ControllerWorker(object):
                  constants.LOADBALANCER: provider_lb,
                  constants.LOADBALANCER_ID: load_balancer.id,
                  constants.PROJECT_ID: db_pool.project_id}
-        self.services_controller.run_poster(
+        self.run_flow(
             flow_utils.get_delete_pool_flow,
             store=store)
 
@@ -673,18 +689,10 @@ class ControllerWorker(object):
                  constants.LOADBALANCER: provider_lb,
                  constants.LOADBALANCER_ID: load_balancer.id,
                  constants.UPDATE_DICT: pool_updates}
-        self.services_controller.run_poster(
+        self.run_flow(
             flow_utils.get_update_pool_flow,
             store=store)
 
-    @tenacity.retry(
-        retry=tenacity.retry_if_exception_type(db_exceptions.NoResultFound),
-        wait=tenacity.wait_incrementing(
-            CONF.haproxy_amphora.api_db_commit_retry_initial_delay,
-            CONF.haproxy_amphora.api_db_commit_retry_backoff,
-            CONF.haproxy_amphora.api_db_commit_retry_max),
-        stop=tenacity.stop_after_attempt(
-            CONF.haproxy_amphora.api_db_commit_retry_attempts))
     def create_l7policy(self, l7policy):
         """Creates an L7 Policy.
 
@@ -692,15 +700,8 @@ class ControllerWorker(object):
         :returns: None
         :raises NoResultFound: Unable to find the object
         """
-        db_l7policy = self._l7policy_repo.get(
-            db_apis.get_session(), id=l7policy[constants.L7POLICY_ID])
-        if not db_l7policy:
-            LOG.warning('Failed to fetch %s %s from DB. Retrying for up to '
-                        '60 seconds.', 'l7policy',
-                        l7policy[constants.L7POLICY_ID])
-            raise db_exceptions.NoResultFound
-
-        db_listener = db_l7policy.listener
+        db_listener = self._listener_repo.get(
+            db_apis.get_session(), id=l7policy[constants.LISTENER_ID])
 
         listeners_dicts = (
             provider_utils.db_listeners_to_provider_dicts_list_of_dicts(
@@ -710,7 +711,7 @@ class ControllerWorker(object):
                  constants.LISTENERS: listeners_dicts,
                  constants.LOADBALANCER_ID: db_listener.load_balancer.id
                  }
-        self.services_controller.run_poster(
+        self.run_flow(
             flow_utils.get_create_l7policy_flow,
             store=store)
 
@@ -731,7 +732,7 @@ class ControllerWorker(object):
                  constants.LISTENERS: listeners_dicts,
                  constants.LOADBALANCER_ID: db_listener.load_balancer.id
                  }
-        self.services_controller.run_poster(
+        self.run_flow(
             flow_utils.get_delete_l7policy_flow,
             store=store)
 
@@ -754,18 +755,10 @@ class ControllerWorker(object):
                  constants.LISTENERS: listeners_dicts,
                  constants.LOADBALANCER_ID: db_listener.load_balancer.id,
                  constants.UPDATE_DICT: l7policy_updates}
-        self.services_controller.run_poster(
+        self.run_flow(
             flow_utils.get_update_l7policy_flow,
             store=store)
 
-    @tenacity.retry(
-        retry=tenacity.retry_if_exception_type(db_exceptions.NoResultFound),
-        wait=tenacity.wait_incrementing(
-            CONF.haproxy_amphora.api_db_commit_retry_initial_delay,
-            CONF.haproxy_amphora.api_db_commit_retry_backoff,
-            CONF.haproxy_amphora.api_db_commit_retry_max),
-        stop=tenacity.stop_after_attempt(
-            CONF.haproxy_amphora.api_db_commit_retry_attempts))
     def create_l7rule(self, l7rule):
         """Creates an L7 Rule.
 
@@ -773,15 +766,8 @@ class ControllerWorker(object):
         :returns: None
         :raises NoResultFound: Unable to find the object
         """
-        db_l7rule = self._l7rule_repo.get(db_apis.get_session(),
-                                          id=l7rule[constants.L7RULE_ID])
-        if not db_l7rule:
-            LOG.warning('Failed to fetch %s %s from DB. Retrying for up to '
-                        '60 seconds.', 'l7rule',
-                        l7rule[constants.L7RULE_ID])
-            raise db_exceptions.NoResultFound
-
-        db_l7policy = db_l7rule.l7policy
+        db_l7policy = self._l7policy_repo.get(db_apis.get_session(),
+                                              id=l7rule[constants.L7POLICY_ID])
 
         load_balancer = db_l7policy.listener.load_balancer
 
@@ -797,7 +783,7 @@ class ControllerWorker(object):
                  constants.LISTENERS: listeners_dicts,
                  constants.LOADBALANCER_ID: load_balancer.id
                  }
-        self.services_controller.run_poster(
+        self.run_flow(
             flow_utils.get_create_l7rule_flow,
             store=store)
 
@@ -823,7 +809,7 @@ class ControllerWorker(object):
                  constants.L7POLICY_ID: db_l7policy.id,
                  constants.LOADBALANCER_ID: load_balancer.id
                  }
-        self.services_controller.run_poster(
+        self.run_flow(
             flow_utils.get_delete_l7rule_flow,
             store=store)
 
@@ -851,7 +837,7 @@ class ControllerWorker(object):
                  constants.L7POLICY_ID: db_l7policy.id,
                  constants.LOADBALANCER_ID: load_balancer.id,
                  constants.UPDATE_DICT: l7rule_updates}
-        self.services_controller.run_poster(
+        self.run_flow(
             flow_utils.get_update_l7rule_flow,
             store=store)
 
@@ -934,8 +920,7 @@ class ControllerWorker(object):
                              constants.LOADBALANCER: provider_lb_dict,
                              constants.SERVER_GROUP_ID: server_group_id,
                              constants.LOADBALANCER_ID: lb_id,
-                             constants.VIP: vip_dict,
-                             constants.AMPHORA_ID: amphora_id}
+                             constants.VIP: vip_dict}
 
             self.services_controller.run_poster(
                 flow_utils.get_failover_amphora_flow,
@@ -1099,7 +1084,7 @@ class ControllerWorker(object):
         except Exception as e:
             with excutils.save_and_reraise_exception(reraise=False):
                 LOG.exception("LB %(lbid)s failover exception: %(exc)s",
-                              {'lbid': load_balancer_id, 'exc': str(e)})
+                              {'lbid': load_balancer_id, 'exc': e})
                 self._lb_repo.update(
                     db_apis.get_session(), load_balancer_id,
                     provisioning_status=constants.ERROR)
@@ -1120,7 +1105,7 @@ class ControllerWorker(object):
         store = {constants.AMPHORA: amp.to_dict(),
                  constants.AMPHORA_ID: amphora_id}
 
-        self.services_controller.run_poster(
+        self.run_flow(
             flow_utils.cert_rotate_amphora_flow,
             store=store)
         LOG.info("Finished amphora cert rotation, amphora's id was: %s",
@@ -1149,7 +1134,7 @@ class ControllerWorker(object):
         store = {constants.AMPHORA: amp.to_dict(),
                  constants.FLAVOR: flavor}
 
-        self.services_controller.run_poster(
+        self.run_flow(
             flow_utils.update_amphora_config_flow,
             store=store)
         LOG.info("Finished amphora agent configuration update, amphora's id "
