@@ -27,6 +27,7 @@ from oslo_log import log as logging
 from oslo_serialization import jsonutils
 from oslo_utils import excutils
 from oslo_utils import uuidutils
+from sqlalchemy.orm import joinedload
 from sqlalchemy.orm import noload
 from sqlalchemy.orm import subqueryload
 from sqlalchemy.sql.expression import false
@@ -106,7 +107,7 @@ class BaseRepository(object):
         """
         with session.begin(subtransactions=True):
             tags = model_kwargs.pop('tags', None)
-            if tags is not None:
+            if tags:
                 resource = session.query(self.model_class).get(id)
                 resource.tags = tags
             session.query(self.model_class).filter_by(
@@ -499,10 +500,52 @@ class Repositories(object):
                     quotas.in_use_member = member_count
                     return False
                 return True
-        except db_exception.DBDeadlock:
+            if _class == data_models.L7Policy:
+                # Decide which quota to use
+                if quotas.l7policy is None:
+                    l7policy_quota = CONF.quotas.default_l7policy_quota
+                else:
+                    l7policy_quota = quotas.l7policy
+                # Get the current in use count
+                if not quotas.in_use_l7policy:
+                    # This is to handle the upgrade case
+                    l7policy_count = session.query(models.L7Policy).filter(
+                        models.L7Policy.project_id == project_id,
+                        models.L7Policy.provisioning_status !=
+                        consts.DELETED).count() + count
+                else:
+                    l7policy_count = quotas.in_use_l7policy + count
+                # Decide if the quota is met
+                if (l7policy_count <= l7policy_quota or
+                        l7policy_quota == consts.QUOTA_UNLIMITED):
+                    quotas.in_use_l7policy = l7policy_count
+                    return False
+                return True
+            if _class == data_models.L7Rule:
+                # Decide which quota to use
+                if quotas.l7rule is None:
+                    l7rule_quota = CONF.quotas.default_l7rule_quota
+                else:
+                    l7rule_quota = quotas.l7rule
+                # Get the current in use count
+                if not quotas.in_use_l7rule:
+                    # This is to handle the upgrade case
+                    l7rule_count = session.query(models.L7Rule).filter(
+                        models.L7Rule.project_id == project_id,
+                        models.L7Rule.provisioning_status !=
+                        consts.DELETED).count() + count
+                else:
+                    l7rule_count = quotas.in_use_l7rule + count
+                # Decide if the quota is met
+                if (l7rule_count <= l7rule_quota or
+                        l7rule_quota == consts.QUOTA_UNLIMITED):
+                    quotas.in_use_l7rule = l7rule_count
+                    return False
+                return True
+        except db_exception.DBDeadlock as e:
             LOG.warning('Quota project lock timed out for project: %(proj)s',
                         {'proj': project_id})
-            raise exceptions.ProjectBusyException()
+            raise exceptions.ProjectBusyException() from e
         return False
 
     def decrement_quota(self, lock_session, _class, project_id, quantity=1):
@@ -584,10 +627,32 @@ class Repositories(object):
                                     'project: %(proj)s that would cause a '
                                     'negative quota.',
                                     {'clss': type(_class), 'proj': project_id})
-        except db_exception.DBDeadlock:
+            if _class == data_models.L7Policy:
+                if (quotas.in_use_l7policy is not None and
+                        quotas.in_use_l7policy > 0):
+                    quotas.in_use_l7policy = (
+                        quotas.in_use_l7policy - quantity)
+                else:
+                    if not CONF.api_settings.auth_strategy == consts.NOAUTH:
+                        LOG.warning('Quota decrement on %(clss)s called on '
+                                    'project: %(proj)s that would cause a '
+                                    'negative quota.',
+                                    {'clss': type(_class), 'proj': project_id})
+            if _class == data_models.L7Rule:
+                if (quotas.in_use_l7rule is not None and
+                        quotas.in_use_l7rule > 0):
+                    quotas.in_use_l7rule = (
+                        quotas.in_use_l7rule - quantity)
+                else:
+                    if not CONF.api_settings.auth_strategy == consts.NOAUTH:
+                        LOG.warning('Quota decrement on %(clss)s called on '
+                                    'project: %(proj)s that would cause a '
+                                    'negative quota.',
+                                    {'clss': type(_class), 'proj': project_id})
+        except db_exception.DBDeadlock as e:
             LOG.warning('Quota project lock timed out for project: %(proj)s',
                         {'proj': project_id})
-            raise exceptions.ProjectBusyException()
+            raise exceptions.ProjectBusyException() from e
 
     def create_load_balancer_tree(self, session, lock_session, lb_dict):
         listener_dicts = lb_dict.pop('listeners', [])
@@ -657,6 +722,13 @@ class Repositories(object):
                     self.sni.create(lock_session, **sni_container)
                 if l7policies_dict:
                     for policy_dict in l7policies_dict:
+                        # Add l7policy quota check
+                        if self.check_quota_met(session,
+                                                lock_session,
+                                                data_models.L7Policy,
+                                                lb_dict['project_id']):
+                            raise exceptions.QuotaException(
+                                resource=data_models.L7Policy._name())
                         l7rules_dict = policy_dict.pop('l7rules')
                         if policy_dict.get('redirect_pool'):
                             # Add pool quota check
@@ -711,6 +783,14 @@ class Repositories(object):
                         policy_dm = self.l7policy.create(lock_session,
                                                          **policy_dict)
                         for rule_dict in l7rules_dict:
+                            # Add l7rule quota check
+                            if self.check_quota_met(
+                                    session,
+                                    lock_session,
+                                    data_models.L7Rule,
+                                    lb_dict['project_id']):
+                                raise exceptions.QuotaException(
+                                    resource=data_models.L7Rule._name())
                             rule_dict['l7policy_id'] = policy_dm.id
                             self.l7rule.create(lock_session, **rule_dict)
             lock_session.commit()
@@ -776,7 +856,7 @@ class LoadBalancerRepository(BaseRepository):
             subqueryload(models.LoadBalancer._tags),
             noload('*'))
 
-        return super(LoadBalancerRepository, self).get_all(
+        return super().get_all(
             session, pagination_helper=pagination_helper,
             query_options=query_options, **filters)
 
@@ -873,7 +953,7 @@ class HealthMonitorRepository(BaseRepository):
             subqueryload(models.HealthMonitor._tags),
             noload('*'))
 
-        return super(HealthMonitorRepository, self).get_all(
+        return super().get_all(
             session, pagination_helper=pagination_helper,
             query_options=query_options, **filters)
 
@@ -944,20 +1024,10 @@ class PoolRepository(BaseRepository):
             subqueryload(models.Pool._tags),
             noload('*'))
 
-        return super(PoolRepository, self).get_all(
+        return super().get_all(
             session, pagination_helper=pagination_helper,
             query_options=query_options, **filters)
 
-    def get_children_count(self, session, pool_id):
-        hm_count = session.query(models.HealthMonitor).filter(
-            models.HealthMonitor.pool_id == pool_id,
-            models.HealthMonitor.provisioning_status != consts.DELETED).count()
-        member_count = session.query(models.Member).filter(
-            models.Member.pool_id == pool_id,
-            models.Member.provisioning_status != consts.DELETED).count()
-
-        return (hm_count, member_count)
-
 
 class MemberRepository(BaseRepository):
     model_class = models.Member
@@ -982,7 +1052,7 @@ class MemberRepository(BaseRepository):
             subqueryload(models.Member._tags),
             noload('*'))
 
-        return super(MemberRepository, self).get_all(
+        return super().get_all(
             session, pagination_helper=pagination_helper,
             query_options=query_options, **filters)
 
@@ -1029,7 +1099,7 @@ class ListenerRepository(BaseRepository):
             subqueryload(models.Listener.allowed_cidrs),
             noload('*'))
 
-        return super(ListenerRepository, self).get_all(
+        return super().get_all(
             session, pagination_helper=pagination_helper,
             query_options=query_options, **filters)
 
@@ -1073,15 +1143,12 @@ class ListenerRepository(BaseRepository):
 
     def update(self, session, id, **model_kwargs):
         with session.begin(subtransactions=True):
-            listener_db = session.query(self.model_class).filter_by(
-                id=id).first()
-            if not listener_db:
-                raise exceptions.NotFound(
-                    resource=data_models.Listener._name(), id=id)
             tags = model_kwargs.pop('tags', None)
-            if tags is not None:
+            if tags:
                 resource = session.query(self.model_class).get(id)
                 resource.tags = tags
+            listener_db = session.query(self.model_class).filter_by(
+                id=id).first()
             # Verify any newly specified default_pool_id exists
             default_pool_id = model_kwargs.get('default_pool_id')
             if default_pool_id:
@@ -1140,24 +1207,67 @@ class ListenerRepository(BaseRepository):
 class ListenerStatisticsRepository(BaseRepository):
     model_class = models.ListenerStatistics
 
-    def replace(self, session, listener_id, amphora_id, **model_kwargs):
-        """replace or insert listener into database."""
+    def replace(self, session, stats_obj):
+        """Create or override a listener's statistics (insert/update)
+
+        :param session: A Sql Alchemy database session
+        :param stats_obj: Listener statistics object to store
+        :type stats_obj: octavia.common.data_models.ListenerStatistics
+        """
+        if not stats_obj.amphora_id:
+            # amphora_id can't be null, so clone the listener_id
+            stats_obj.amphora_id = stats_obj.listener_id
+
         with session.begin(subtransactions=True):
+            # TODO(johnsom): This can be simplified/optimized using an "upsert"
             count = session.query(self.model_class).filter_by(
-                listener_id=listener_id, amphora_id=amphora_id).count()
+                listener_id=stats_obj.listener_id,
+                amphora_id=stats_obj.amphora_id).count()
             if count:
                 session.query(self.model_class).filter_by(
-                    listener_id=listener_id,
-                    amphora_id=amphora_id).update(
-                    model_kwargs,
+                    listener_id=stats_obj.listener_id,
+                    amphora_id=stats_obj.amphora_id).update(
+                    stats_obj.get_stats(),
                     synchronize_session=False)
             else:
-                model_kwargs['listener_id'] = listener_id
-                model_kwargs['amphora_id'] = amphora_id
-                self.create(session, **model_kwargs)
+                self.create(session, **stats_obj.db_fields())
+
+    def increment(self, session, delta_stats):
+        """Updates a listener's statistics, incrementing by the passed deltas.
+
+        :param session: A Sql Alchemy database session
+        :param delta_stats: Listener statistics deltas to add
+        :type delta_stats: octavia.common.data_models.ListenerStatistics
+        """
+        if not delta_stats.amphora_id:
+            # amphora_id can't be null, so clone the listener_id
+            delta_stats.amphora_id = delta_stats.listener_id
+
+        with session.begin(subtransactions=True):
+            # TODO(johnsom): This can be simplified/optimized using an "upsert"
+            count = session.query(self.model_class).filter_by(
+                listener_id=delta_stats.listener_id,
+                amphora_id=delta_stats.amphora_id).count()
+            if count:
+                existing_stats = session.query(
+                    self.model_class).with_for_update().filter_by(
+                    listener_id=delta_stats.listener_id,
+                    amphora_id=delta_stats.amphora_id).one()
+                existing_stats += delta_stats
+                existing_stats.active_connections = (
+                    delta_stats.active_connections)
+            else:
+                self.create(session, **delta_stats.db_fields())
 
     def update(self, session, listener_id, **model_kwargs):
-        """Updates a listener's statistics by a listener's id."""
+        """Updates a listener's statistics, overriding with the passed values.
+
+        :param session: A Sql Alchemy database session
+        :param listener_id: The UUID of the listener to update
+        :type listener_id: str
+        :param model_kwargs: Entity attributes that should be updated
+
+        """
         with session.begin(subtransactions=True):
             session.query(self.model_class).filter_by(
                 listener_id=listener_id).update(model_kwargs)
@@ -1185,7 +1295,7 @@ class AmphoraRepository(BaseRepository):
             subqueryload(models.Amphora.load_balancer),
             noload('*'))
 
-        return super(AmphoraRepository, self).get_all(
+        return super().get_all(
             session, pagination_helper=pagination_helper,
             query_options=query_options, **filters)
 
@@ -1259,7 +1369,10 @@ class AmphoraRepository(BaseRepository):
                 .filter(models.Amphora.status != consts.DELETED)
                 # And the LB is also not DELETED
                 .filter(models.LoadBalancer.provisioning_status !=
-                        consts.DELETED)).first()
+                        consts.DELETED)
+                # And what does this do? Some SQLAlchemy magic?
+                .options(joinedload('*'))
+            ).first()
             if db_lb:
                 return db_lb.to_data_model()
             return None
@@ -1395,6 +1508,27 @@ class AmphoraRepository(BaseRepository):
 
         return lb
 
+    def test_and_set_status_for_delete(self, lock_session, id):
+        """Tests and sets an amphora status.
+
+        Puts a lock on the amphora table to check the status of the
+        amphora. The status must be either AMPHORA_READY or ERROR to
+        successfuly update the amphora status.
+
+        :param lock_session: A Sql Alchemy database session.
+        :param id: id of Load Balancer
+        :raises ImmutableObject: The amphora is not in a state that can be
+                                 deleted.
+        :raises NoResultFound: The amphora was not found or already deleted.
+        :returns: None
+        """
+        amp = lock_session.query(self.model_class).with_for_update().filter_by(
+            id=id).filter(self.model_class.status != consts.DELETED).one()
+        if amp.status not in [consts.AMPHORA_READY, consts.ERROR]:
+            raise exceptions.ImmutableObject(resource=consts.AMPHORA, id=id)
+        amp.status = consts.PENDING_DELETE
+        lock_session.flush()
+
 
 class AmphoraBuildReqRepository(BaseRepository):
     model_class = models.AmphoraBuildRequest
@@ -1594,7 +1728,7 @@ class L7RuleRepository(BaseRepository):
             subqueryload(models.L7Rule._tags),
             noload('*'))
 
-        return super(L7RuleRepository, self).get_all(
+        return super().get_all(
             session, pagination_helper=pagination_helper,
             query_options=query_options, **filters)
 
@@ -1866,6 +2000,8 @@ class QuotasRepository(BaseRepository):
             quotas.listener = None
             quotas.member = None
             quotas.pool = None
+            quotas.l7policy = None
+            quotas.l7rule = None
             session.flush()
 
 
