@@ -16,6 +16,7 @@ import time
 
 from neutronclient.common import exceptions as neutron_client_exceptions
 from novaclient import exceptions as nova_client_exceptions
+from octavia_lib.common import constants as lib_consts
 from oslo_config import cfg
 from oslo_log import log as logging
 from stevedore import driver as stevedore_driver
@@ -41,7 +42,7 @@ CONF = cfg.CONF
 class AllowedAddressPairsDriver(neutron_base.BaseNeutronDriver):
 
     def __init__(self):
-        super(AllowedAddressPairsDriver, self).__init__()
+        super().__init__()
         self._check_aap_loaded()
         self.compute = stevedore_driver.DriverManager(
             namespace='octavia.compute.drivers',
@@ -95,15 +96,15 @@ class AllowedAddressPairsDriver(neutron_base.BaseNeutronDriver):
             LOG.debug('Created vip port: %(port_id)s for amphora: %(amp)s',
                       {'port_id': new_port.id, 'amp': amphora.id})
 
-        except Exception:
+        except Exception as e:
             message = _('Error creating the base (VRRP) port for the VIP with '
                         'port details: {}').format(port)
             LOG.exception(message)
-            raise base.PlugVIPException(message)
+            raise base.PlugVIPException(message) from e
 
         try:
             interface = self.plug_port(amphora, new_port)
-        except Exception:
+        except Exception as e:
             message = _('Error plugging amphora (compute_id: {compute_id}) '
                         'into vip network {network_id}.').format(
                             compute_id=amphora.compute_id,
@@ -119,7 +120,7 @@ class AllowedAddressPairsDriver(neutron_base.BaseNeutronDriver):
                               'plug_port failed. This resource is being '
                               'abandoned and should be manually deleted when '
                               'neutron is functional.', new_port.id)
-            raise base.PlugVIPException(message)
+            raise base.PlugVIPException(message) from e
         return interface
 
     def _add_vip_address_pair(self, port_id, vip_address):
@@ -127,12 +128,12 @@ class AllowedAddressPairsDriver(neutron_base.BaseNeutronDriver):
             self._add_allowed_address_pair_to_port(port_id, vip_address)
         except neutron_client_exceptions.PortNotFoundClient as e:
             raise base.PortNotFound(str(e))
-        except Exception:
+        except Exception as e:
             message = _('Error adding allowed address pair {ip} '
                         'to port {port_id}.').format(ip=vip_address,
                                                      port_id=port_id)
             LOG.exception(message)
-            raise base.PlugVIPException(message)
+            raise base.PlugVIPException(message) from e
 
     def _get_lb_security_group(self, load_balancer_id):
         sec_grp_name = common_utils.get_vip_security_group_name(
@@ -151,7 +152,6 @@ class AllowedAddressPairsDriver(neutron_base.BaseNeutronDriver):
             security_group_id=sec_grp_id)
 
         updated_ports = []
-        listener_peer_ports = []
         for listener in load_balancer.listeners:
             if (listener.provisioning_status in [constants.PENDING_DELETE,
                                                  constants.DELETED]):
@@ -160,6 +160,8 @@ class AllowedAddressPairsDriver(neutron_base.BaseNeutronDriver):
             protocol = constants.PROTOCOL_TCP.lower()
             if listener.protocol == constants.PROTOCOL_UDP:
                 protocol = constants.PROTOCOL_UDP.lower()
+            elif listener.protocol == lib_consts.PROTOCOL_SCTP:
+                protocol = lib_consts.PROTOCOL_SCTP.lower()
 
             if listener.allowed_cidrs:
                 for ac in listener.allowed_cidrs:
@@ -169,17 +171,11 @@ class AllowedAddressPairsDriver(neutron_base.BaseNeutronDriver):
                 port = (listener.protocol_port, protocol, None)
                 updated_ports.append(port)
 
-            listener_peer_ports.append(listener.peer_port)
-
-        # As the peer port will hold the tcp connection for keepalived and
-        # haproxy session synchronization, so here the security group rule
-        # should be just related with tcp protocol only. To avoid adding
-        # duplicate rules, peer_port info should be added if updated_ports
-        # does not have the peer_port entry with allowed_cidr 0.0.0.0/0
-        tcp_lower = constants.PROTOCOL_TCP.lower()
-        for peer_port in listener_peer_ports:
-            if (peer_port, tcp_lower, "0.0.0.0/0") not in updated_ports:
-                updated_ports.append((peer_port, tcp_lower, None))
+            # As the peer port will hold the tcp connection for keepalived and
+            # haproxy session synchronization, so here the security group rule
+            # should be just related with tcp protocol only.
+            updated_ports.append(
+                (listener.peer_port, constants.PROTOCOL_TCP.lower(), None))
 
         # Just going to use port_range_max for now because we can assume that
         # port_range_max and min will be the same since this driver is
@@ -190,7 +186,8 @@ class AllowedAddressPairsDriver(neutron_base.BaseNeutronDriver):
             # None ports with the egress rules.  VRRP uses protocol 51 and 112
             if (rule.get('direction') == 'egress' or
                 rule.get('protocol').upper() not in
-                    [constants.PROTOCOL_TCP, constants.PROTOCOL_UDP]):
+                    [constants.PROTOCOL_TCP, constants.PROTOCOL_UDP,
+                     lib_consts.PROTOCOL_SCTP]):
                 continue
             old_ports.append((rule.get('port_range_max'),
                               rule.get('protocol').lower(),
@@ -352,9 +349,9 @@ class AllowedAddressPairsDriver(neutron_base.BaseNeutronDriver):
         try:
             port = self.get_port(vip.port_id)
         except base.PortNotFound:
-            LOG.warning("Can't deallocate VIP because the vip port %s "
+            LOG.warning("Can't deallocate VIP because the vip port {0} "
                         "cannot be found in neutron. "
-                        "Continuing cleanup.", vip.port_id)
+                        "Continuing cleanup.".format(vip.port_id))
             port = None
 
         self._delete_security_group(vip, port)
@@ -366,11 +363,11 @@ class AllowedAddressPairsDriver(neutron_base.BaseNeutronDriver):
                     neutron_client_exceptions.PortNotFoundClient):
                 LOG.debug('VIP port %s already deleted. Skipping.',
                           vip.port_id)
-            except Exception:
+            except Exception as e:
                 message = _('Error deleting VIP port_id {port_id} from '
                             'neutron').format(port_id=vip.port_id)
                 LOG.exception(message)
-                raise base.DeallocateVIPException(message)
+                raise base.DeallocateVIPException(message) from e
         elif port:
             LOG.info("Port %s will not be deleted by Octavia as it was "
                      "not created by Octavia.", vip.port_id)
@@ -550,12 +547,12 @@ class AllowedAddressPairsDriver(neutron_base.BaseNeutronDriver):
             }}
             self.neutron_client.update_port(interface.port_id,
                                             aap_update)
-        except Exception:
+        except Exception as e:
             message = _('Error unplugging VIP. Could not clear '
                         'allowed address pairs from port '
                         '{port_id}.').format(port_id=vip.port_id)
             LOG.exception(message)
-            raise base.UnplugVIPException(message)
+            raise base.UnplugVIPException(message) from e
 
         # Delete the VRRP port if we created it
         try:
@@ -568,16 +565,16 @@ class AllowedAddressPairsDriver(neutron_base.BaseNeutronDriver):
         except Exception as e:
             LOG.error('Failed to delete port.  Resources may still be in '
                       'use for port: %(port)s due to error: %(except)s',
-                      {constants.PORT: amphora.vrrp_port_id, 'except': str(e)})
+                      {constants.PORT: amphora.vrrp_port_id, 'except': e})
 
     def unplug_vip(self, load_balancer, vip):
         try:
             subnet = self.get_subnet(vip.subnet_id)
-        except base.SubnetNotFound:
+        except base.SubnetNotFound as e:
             msg = ("Can't unplug vip because vip subnet {0} was not "
                    "found").format(vip.subnet_id)
             LOG.exception(msg)
-            raise base.PluggedVIPNotFound(msg)
+            raise base.PluggedVIPNotFound(msg) from e
         for amphora in filter(
                 lambda amp: amp.status == constants.AMPHORA_ALLOCATED,
                 load_balancer.amphorae):
@@ -594,13 +591,13 @@ class AllowedAddressPairsDriver(neutron_base.BaseNeutronDriver):
             if 'Network' in str(e):
                 raise base.NetworkNotFound(str(e))
             raise base.PlugNetworkException(str(e))
-        except Exception:
+        except Exception as e:
             message = _('Error plugging amphora (compute_id: {compute_id}) '
                         'into network {network_id}.').format(
                             compute_id=compute_id,
                             network_id=network_id)
             LOG.exception(message)
-            raise base.PlugNetworkException(message)
+            raise base.PlugNetworkException(message) from e
 
         return self._nova_interface_to_octavia_interface(compute_id, interface)
 
@@ -626,8 +623,9 @@ class AllowedAddressPairsDriver(neutron_base.BaseNeutronDriver):
             raise exceptions.MissingVIPSecurityGroup(lb_id=load_balancer.id)
         else:
             LOG.warning('VIP security group missing when updating the VIP for '
-                        'delete on load balancer: %s. Skipping update '
-                        'because this is for delete.', load_balancer.id)
+                        'delete on load balancer: {lb_id}. Skipping update '
+                        'because this is for delete.'.format(
+                            lb_id=load_balancer.id))
 
     def failover_preparation(self, amphora):
         if self.dns_integration_enabled:
@@ -653,8 +651,8 @@ class AllowedAddressPairsDriver(neutron_base.BaseNeutronDriver):
                     port.id, {constants.PORT: {'dns_name': ''}})
 
             except (neutron_client_exceptions.NotFound,
-                    neutron_client_exceptions.PortNotFoundClient):
-                raise base.PortNotFound()
+                    neutron_client_exceptions.PortNotFoundClient) as e:
+                raise base.PortNotFound() from e
 
     def plug_port(self, amphora, port):
         try:
@@ -677,14 +675,14 @@ class AllowedAddressPairsDriver(neutron_base.BaseNeutronDriver):
                 network_id=port.network_id,
                 port_id=port.id,
                 fixed_ips=port.fixed_ips)
-        except Exception:
+        except Exception as e:
             message = _('Error plugging amphora (compute_id: '
                         '{compute_id}) into port '
                         '{port_id}.').format(
                             compute_id=amphora.compute_id,
                             port_id=port.id)
             LOG.exception(message)
-            raise base.PlugNetworkException(message)
+            raise base.PlugNetworkException(message) from e
 
         return plugged_interface
 
