@@ -31,12 +31,12 @@ class TestAvailabilityZones(base.BaseAPITest):
     root_tag_links = 'availability_zones_links'
 
     def setUp(self):
-        super(TestAvailabilityZones, self).setUp()
+        super().setUp()
         self.azp = self.create_availability_zone_profile(
             'test1', 'noop_driver', '{"compute_zone": "my_az_1"}')
 
     def _assert_request_matches_response(self, req, resp, **optionals):
-        self.assertNotIn('id', resp)  # AZs do not expose an ID
+        self.assertTrue('id' not in resp)  # AZs do not expose an ID
         req_description = req.get('description')
         self.assertEqual(req.get('name'), resp.get('name'))
         if not req_description:
