@@ -26,7 +26,7 @@ from octavia.tests.unit import base
 
 class TestAmphoraDriver(base.TestRpc):
     def setUp(self):
-        super(TestAmphoraDriver, self).setUp()
+        super().setUp()
         self.amp_driver = driver.AmphoraProviderDriver()
         self.sample_data = sample_data_models.SampleDriverDataModels()
 
@@ -42,21 +42,6 @@ class TestAmphoraDriver(base.TestRpc):
 
         self.assertEqual(self.sample_data.provider_vip_dict, provider_vip_dict)
 
-    @mock.patch('octavia.common.utils.get_network_driver')
-    def test_create_vip_port_without_port_security_enabled(
-            self, mock_get_net_driver):
-        mock_net_driver = mock.MagicMock()
-        mock_get_net_driver.return_value = mock_net_driver
-        network = mock.MagicMock()
-        network.port_security_enabled = False
-        mock_net_driver.get_network.return_value = network
-        mock_net_driver.allocate_vip.return_value = self.sample_data.db_vip
-
-        self.assertRaises(exceptions.DriverError,
-                          self.amp_driver.create_vip_port,
-                          self.sample_data.lb_id, self.sample_data.project_id,
-                          self.sample_data.provider_vip_dict)
-
     @mock.patch('octavia.common.utils.get_network_driver')
     def test_create_vip_port_failed(self, mock_get_net_driver):
         mock_net_driver = mock.MagicMock()
@@ -137,11 +122,23 @@ class TestAmphoraDriver(base.TestRpc):
     @mock.patch('oslo_messaging.RPCClient.cast')
     def test_listener_create(self, mock_cast):
         provider_listener = driver_dm.Listener(
-            listener_id=self.sample_data.listener1_id)
+            listener_id=self.sample_data.listener1_id,
+            alpn_protocols=consts.AMPHORA_SUPPORTED_ALPN_PROTOCOLS)
         self.amp_driver.listener_create(provider_listener)
         payload = {consts.LISTENER: provider_listener.to_dict()}
         mock_cast.assert_called_with({}, 'create_listener', **payload)
 
+    @mock.patch('oslo_messaging.RPCClient.cast')
+    def test_listener_create_unsupported_alpn(self, mock_cast):
+        provider_listener = driver_dm.Listener(
+            listener_id=self.sample_data.listener1_id)
+        provider_listener.alpn_protocols = ['http/1.1', 'eureka']
+        self.assertRaises(
+            exceptions.UnsupportedOptionError,
+            self.amp_driver.listener_create,
+            provider_listener)
+        mock_cast.assert_not_called()
+
     @mock.patch('oslo_messaging.RPCClient.cast')
     def test_listener_delete(self, mock_cast):
         provider_listener = driver_dm.Listener(
@@ -178,6 +175,19 @@ class TestAmphoraDriver(base.TestRpc):
                    consts.LISTENER_UPDATES: listener_dict}
         mock_cast.assert_called_with({}, 'update_listener', **payload)
 
+    @mock.patch('oslo_messaging.RPCClient.cast')
+    def test_listener_update_unsupported_alpn(self, mock_cast):
+        old_provider_listener = driver_dm.Listener(
+            listener_id=self.sample_data.listener1_id)
+        provider_listener = driver_dm.Listener(
+            listener_id=self.sample_data.listener1_id,
+            alpn_protocols=['http/1.1', 'eureka'])
+        self.assertRaises(
+            exceptions.UnsupportedOptionError,
+            self.amp_driver.listener_update,
+            old_provider_listener,
+            provider_listener)
+
     # Pool
     @mock.patch('oslo_messaging.RPCClient.cast')
     def test_pool_create(self, mock_cast):
@@ -212,19 +222,8 @@ class TestAmphoraDriver(base.TestRpc):
         old_provider_pool = driver_dm.Pool(
             pool_id=self.sample_data.pool1_id)
         provider_pool = driver_dm.Pool(
-            pool_id=self.sample_data.pool1_id, admin_state_up=True,
-            ca_tls_container_data='CA DATA', ca_tls_container_ref='CA REF',
-            crl_container_data='CRL DATA', crl_container_ref='CRL REF',
-            description='TEST DESCRIPTION', name='TEST NAME',
-            lb_algorithm=consts.LB_ALGORITHM_SOURCE_IP,
-            session_persistence='FAKE SP', tls_container_data='TLS DATA',
-            tls_container_ref='TLS REF', tls_enabled=False)
-        pool_dict = {'description': 'TEST DESCRIPTION',
-                     'lb_algorithm': 'SOURCE_IP', 'name': 'TEST NAME',
-                     'session_persistence': 'FAKE SP', 'tls_enabled': False,
-                     'enabled': True, 'tls_certificate_id': 'TLS REF',
-                     'ca_tls_certificate_id': 'CA REF',
-                     'crl_container_id': 'CRL REF'}
+            pool_id=self.sample_data.pool1_id, admin_state_up=True)
+        pool_dict = {'enabled': True}
         self.amp_driver.pool_update(old_provider_pool, provider_pool)
         payload = {consts.ORIGINAL_POOL: old_provider_pool.to_dict(),
                    consts.POOL_UPDATES: pool_dict}
@@ -429,22 +428,13 @@ class TestAmphoraDriver(base.TestRpc):
     @mock.patch('oslo_messaging.RPCClient.cast')
     def test_member_batch_update_clear_already_empty(
             self, mock_cast, mock_pool_get, mock_session):
-        """Expect that we will pass an empty payload if directed.
-
-        Logic for whether or not to attempt this will be done above the driver
-        layer, so our driver is responsible to forward the request even if it
-        is a perceived no-op.
-        """
         mock_pool = mock.MagicMock()
         mock_pool_get.return_value = mock_pool
 
         self.amp_driver.member_batch_update(
             self.sample_data.pool1_id, [])
 
-        payload = {'old_members': [],
-                   'new_members': [],
-                   'updated_members': []}
-        mock_cast.assert_called_with({}, 'batch_update_members', **payload)
+        mock_cast.assert_not_called()
 
     # Health Monitor
     @mock.patch('oslo_messaging.RPCClient.cast')
@@ -721,16 +711,20 @@ class TestAmphoraDriver(base.TestRpc):
 
     def test_validate_availability_zone(self):
         # Test compute zone
-        ref_dict = {consts.COMPUTE_ZONE: 'my_compute_zone'}
-        self.amp_driver.validate_availability_zone(ref_dict)
+        with mock.patch('stevedore.driver.DriverManager.driver') as m_driver:
+            m_driver.validate_availability_zone.return_value = None
+            ref_dict = {consts.COMPUTE_ZONE: 'my_compute_zone'}
+            self.amp_driver.validate_availability_zone(ref_dict)
 
-        # Test vip networks
-        ref_dict = {consts.VALID_VIP_NETWORKS: ['my_vip_net']}
-        self.amp_driver.validate_availability_zone(ref_dict)
+        with mock.patch('octavia.common.utils.get_network_driver') as m_driver:
+            # Test vip networks
+            m_driver.get_network.return_value = None
+            ref_dict = {consts.VALID_VIP_NETWORKS: ['my_vip_net']}
+            self.amp_driver.validate_availability_zone(ref_dict)
 
-        # Test management network
-        ref_dict = {consts.MANAGEMENT_NETWORK: 'my_management_net'}
-        self.amp_driver.validate_availability_zone(ref_dict)
+            # Test management network
+            ref_dict = {consts.MANAGEMENT_NETWORK: 'my_management_net'}
+            self.amp_driver.validate_availability_zone(ref_dict)
 
         # Test bad availability zone metadata key
         ref_dict = {'bogus': 'bogus'}
@@ -750,9 +744,9 @@ class TestAmphoraDriver(base.TestRpc):
     def test_encrypt_listener_dict(self, mock_fernet):
         mock_fern = mock.MagicMock()
         mock_fernet.return_value = mock_fern
-        TEST_DATA = {'cert': b'some data'}
-        TEST_DATA2 = {'test': 'more data'}
-        FAKE_ENCRYPTED_DATA = b'alqwkhjetrhth'
+        TEST_DATA = 'some data'
+        TEST_DATA2 = 'more data'
+        FAKE_ENCRYPTED_DATA = 'alqwkhjetrhth'
         mock_fern.encrypt.return_value = FAKE_ENCRYPTED_DATA
 
         # We need a class instance with the mock
@@ -763,21 +757,21 @@ class TestAmphoraDriver(base.TestRpc):
 
         amp_driver._encrypt_listener_dict(list_dict)
 
-        mock_fern.encrypt.assert_called_once_with(b'some data')
+        mock_fern.encrypt.assert_called_once_with(TEST_DATA)
 
-        self.assertEqual({'cert': FAKE_ENCRYPTED_DATA},
+        self.assertEqual(FAKE_ENCRYPTED_DATA,
                          list_dict[consts.DEFAULT_TLS_CONTAINER_DATA])
 
         mock_fern.reset_mock()
 
         # Test just sni_container_data
-        TEST_DATA = {'cert': b'some data'}
-        sni_dict = {consts.SNI_CONTAINER_DATA: [TEST_DATA, TEST_DATA2]}
+        list_dict = {consts.SNI_CONTAINER_DATA: [TEST_DATA, TEST_DATA2]}
 
-        amp_driver._encrypt_listener_dict(sni_dict)
+        amp_driver._encrypt_listener_dict(list_dict)
 
-        mock_fern.encrypt.assert_called_once_with(b'some data')
+        calls = [mock.call(TEST_DATA), mock.call(TEST_DATA2)]
 
-        encrypted_sni = [{'cert': FAKE_ENCRYPTED_DATA},
-                         TEST_DATA2]
-        self.assertEqual(encrypted_sni, sni_dict[consts.SNI_CONTAINER_DATA])
+        mock_fern.encrypt.assert_has_calls(calls)
+
+        encrypted_sni = [FAKE_ENCRYPTED_DATA, FAKE_ENCRYPTED_DATA]
+        self.assertEqual(encrypted_sni, list_dict[consts.SNI_CONTAINER_DATA])
