@@ -117,8 +117,9 @@ RET_MEMBER_3 = {
 RET_POOL_1 = {
     'id': 'sample_pool_id_1',
     'protocol': 'http',
-    'proxy_protocol': False,
+    'proxy_protocol': None,
     'lb_algorithm': 'roundrobin',
+    'listener_tls_enabled': False,
     'members': [RET_MEMBER_1, RET_MEMBER_2],
     'health_monitor': RET_MONITOR_1,
     'session_persistence': RET_PERSISTENCE,
@@ -134,8 +135,9 @@ RET_POOL_1 = {
 RET_POOL_2 = {
     'id': 'sample_pool_id_2',
     'protocol': 'http',
-    'proxy_protocol': False,
+    'proxy_protocol': None,
     'lb_algorithm': 'roundrobin',
+    'listener_tls_enabled': False,
     'members': [RET_MEMBER_3],
     'health_monitor': RET_MONITOR_2,
     'session_persistence': RET_PERSISTENCE,
@@ -468,7 +470,7 @@ RET_UDP_POOL = {
     'id': 'sample_pool_id_1',
     'enabled': True,
     'health_monitor': UDP_HEALTH_MONITOR_NO_SCRIPT,
-    'lb_algorithm': 'wrr',
+    'lb_algorithm': 'rr',
     'members': [UDP_MEMBER_1, UDP_MEMBER_2],
     'protocol': 'udp',
     'session_persistence': UDP_SOURCE_IP_BODY
@@ -480,7 +482,7 @@ RET_UDP_LISTENER = {
         'id': 'sample_pool_id_1',
         'enabled': True,
         'health_monitor': RET_UDP_HEALTH_MONITOR,
-        'lb_algorithm': 'wrr',
+        'lb_algorithm': 'rr',
         'members': [UDP_MEMBER_1, UDP_MEMBER_2],
         'protocol': 'udp',
         'session_persistence': UDP_SOURCE_IP_BODY
@@ -603,13 +605,20 @@ def sample_listener_tuple(proto=None, monitor=True, alloc_default_pool=True,
                           provisioning_status=constants.ACTIVE,
                           tls_ciphers=constants.CIPHERS_OWASP_SUITE_B,
                           backend_tls_ciphers=None,
-                          sample_default_pool=1,
-                          pool_enabled=True):
+                          tls_versions=constants.TLS_VERSIONS_OWASP_SUITE_B,
+                          backend_tls_versions=constants.
+                          TLS_VERSIONS_OWASP_SUITE_B,
+                          alpn_protocols=constants.
+                          AMPHORA_SUPPORTED_ALPN_PROTOCOLS):
     proto = 'HTTP' if proto is None else proto
     if be_proto is None:
         be_proto = 'HTTP' if proto == 'TERMINATED_HTTPS' else proto
     if proto != constants.PROTOCOL_TERMINATED_HTTPS:
         tls_ciphers = None
+        tls_versions = None
+        alpn_protocols = None
+    if pool_cert is False:
+        backend_tls_versions = None
     topology = 'SINGLE' if topology is None else topology
     port = '443' if proto in ['HTTPS', 'TERMINATED_HTTPS'] else '80'
     peer_port = 1024 if peer_port is None else peer_port
@@ -624,7 +633,7 @@ def sample_listener_tuple(proto=None, monitor=True, alloc_default_pool=True,
                     'timeout_tcp_inspect, client_ca_tls_certificate_id, '
                     'client_ca_tls_certificate, client_authentication, '
                     'client_crl_container_id, provisioning_status, '
-                    'tls_ciphers')
+                    'tls_ciphers, tls_versions, alpn_protocols')
     if l7:
         pools = [
             sample_pool_tuple(
@@ -637,7 +646,7 @@ def sample_listener_tuple(proto=None, monitor=True, alloc_default_pool=True,
                 hm_host_http_check=hm_host_http_check,
                 listener_id='sample_listener_id_1',
                 tls_ciphers=backend_tls_ciphers,
-                enabled=pool_enabled),
+                tls_versions=backend_tls_versions),
             sample_pool_tuple(
                 proto=be_proto, monitor=monitor, persistence=persistence,
                 persistence_type=persistence_type,
@@ -648,7 +657,7 @@ def sample_listener_tuple(proto=None, monitor=True, alloc_default_pool=True,
                 hm_host_http_check=hm_host_http_check,
                 listener_id='sample_listener_id_1',
                 tls_ciphers=backend_tls_ciphers,
-                enabled=pool_enabled)]
+                tls_versions=None)]
         l7policies = [
             sample_l7policy_tuple('sample_l7policy_id_1', sample_policy=1),
             sample_l7policy_tuple('sample_l7policy_id_2', sample_policy=2),
@@ -673,7 +682,7 @@ def sample_listener_tuple(proto=None, monitor=True, alloc_default_pool=True,
                 hm_host_http_check=hm_host_http_check,
                 listener_id='sample_listener_id_1',
                 tls_ciphers=backend_tls_ciphers,
-                enabled=pool_enabled)]
+                tls_versions=backend_tls_versions)]
         l7policies = []
     listener = in_listener(
         id=id,
@@ -697,9 +706,7 @@ def sample_listener_tuple(proto=None, monitor=True, alloc_default_pool=True,
             pool_ca_cert=pool_ca_cert,
             pool_crl=pool_crl,
             tls_enabled=tls_enabled,
-            hm_host_http_check=hm_host_http_check,
-            sample_pool=sample_default_pool,
-            enabled=pool_enabled
+            hm_host_http_check=hm_host_http_check
         ) if alloc_default_pool else '',
         connection_limit=connection_limit,
         tls_certificate_id='cont_id_1' if tls else '',
@@ -744,7 +751,9 @@ def sample_listener_tuple(proto=None, monitor=True, alloc_default_pool=True,
             constants.CLIENT_AUTH_NONE),
         client_crl_container_id='cont_id_crl' if client_crl_cert else '',
         provisioning_status=provisioning_status,
-        tls_ciphers=tls_ciphers
+        tls_ciphers=tls_ciphers,
+        tls_versions=tls_versions,
+        alpn_protocols=alpn_protocols
     )
     if recursive_nest:
         listener.load_balancer.listeners.append(listener)
@@ -785,17 +794,19 @@ def sample_pool_tuple(listener_id=None, proto=None, monitor=True,
                       hm_host_http_check=False,
                       provisioning_status=constants.ACTIVE,
                       tls_ciphers=constants.CIPHERS_OWASP_SUITE_B,
-                      enabled=True):
+                      tls_versions=constants.TLS_VERSIONS_OWASP_SUITE_B,
+                      lb_algorithm=constants.LB_ALGORITHM_ROUND_ROBIN):
     proto = 'HTTP' if proto is None else proto
     if not tls_enabled:
         tls_ciphers = None
+        tls_versions = None
     monitor_proto = proto if monitor_proto is None else monitor_proto
     in_pool = collections.namedtuple(
         'pool', 'id, protocol, lb_algorithm, members, health_monitor, '
                 'session_persistence, enabled, operating_status, '
                 'tls_certificate_id, ca_tls_certificate_id, '
                 'crl_container_id, tls_enabled, tls_ciphers, '
-                'provisioning_status, ' +
+                'tls_versions, provisioning_status, ' +
                 constants.HTTP_REUSE)
     if (proto == constants.PROTOCOL_UDP and
             persistence_type == constants.SESSION_PERSISTENCE_SOURCE_IP):
@@ -807,15 +818,7 @@ def sample_pool_tuple(listener_id=None, proto=None, monitor=True,
                   'persistence_cookie': persistence_cookie}
     persis = sample_session_persistence_tuple(**kwargs)
     mon = None
-    if sample_pool == 0:
-        id = 'sample_pool_id_0'
-        members = []
-        if monitor is True:
-            mon = sample_health_monitor_tuple(
-                proto=monitor_proto,
-                host_http_check=hm_host_http_check,
-                expected_codes=monitor_expected_codes)
-    elif sample_pool == 1:
+    if sample_pool == 1:
         id = 'sample_pool_id_1'
         members = [sample_member_tuple('sample_member_id_1', '10.0.0.99',
                                        monitor_ip_port=monitor_ip_port),
@@ -840,17 +843,18 @@ def sample_pool_tuple(listener_id=None, proto=None, monitor=True,
     return in_pool(
         id=id,
         protocol=proto,
-        lb_algorithm='ROUND_ROBIN',
+        lb_algorithm=lb_algorithm,
         members=members,
         health_monitor=mon,
         session_persistence=persis if persistence is True else None,
-        enabled=enabled,
+        enabled=True,
         operating_status='ACTIVE', has_http_reuse=has_http_reuse,
         tls_certificate_id='pool_cont_1' if pool_cert else None,
         ca_tls_certificate_id='pool_ca_1' if pool_ca_cert else None,
         crl_container_id='pool_crl' if pool_crl else None,
         tls_enabled=tls_enabled,
         tls_ciphers=tls_ciphers,
+        tls_versions=tls_versions,
         provisioning_status=provisioning_status)
 
 
